from abqpy.decorators import abaqus_class_doc

from ..Region.Region import Region


@abaqus_class_doc
class GeometricRestriction:
    """The GeometricRestriction object is the abstract base type for other GeometricRestriction objects. The
    GeometricRestriction object has no explicit constructor. The methods and members of the GeometricRestriction
    object are common to all objects derived from GeometricRestriction.

    .. note::
        This object can be accessed by::

            import optimization
            mdb.models[name].optimizationTasks[name].geometricRestrictions[name]
    """

    #: A String specifying the geometric restriction repository key.
    name: str = ""

    #: A Region object specifying the region to which the geometric restriction is applied.
    #: When used with a TopologyTask, there is no default value. When used with a ShapeTask,
    #: the default value is MODEL.
    region: Region = Region()



from __future__ import annotations

from typing import Union

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Amplitude.Correlation import Correlation
from ..Model.ModelBase import ModelBase
from ..Region.Region import Region
from ..Region.RegionArray import RegionArray
from ..UtilityAndView.abaqusConstants import (
    FREE,
    INFLOW,
    NOT_APPLICABLE,
    OFF,
    ON,
    UNIFORM,
    UNSET,
    ZERO_PRESSURE,
    Boolean,
)
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .AccelerationBaseMotionBC import AccelerationBaseMotionBC
from .AccelerationBaseMotionBCState import AccelerationBaseMotionBCState
from .AccelerationBC import AccelerationBC
from .AccelerationBCState import AccelerationBCState
from .AcousticPressureBC import AcousticPressureBC
from .AcousticPressureBCState import AcousticPressureBCState
from .ConcentrationBC import ConcentrationBC
from .ConcentrationBCState import ConcentrationBCState
from .ConnAccelerationBC import ConnAccelerationBC
from .ConnAccelerationBCState import ConnAccelerationBCState
from .ConnDisplacementBC import ConnDisplacementBC
from .ConnDisplacementBCState import ConnDisplacementBCState
from .ConnVelocityBC import ConnVelocityBC
from .ConnVelocityBCState import ConnVelocityBCState
from .DisplacementBaseMotionBC import DisplacementBaseMotionBC
from .DisplacementBaseMotionBCState import DisplacementBaseMotionBCState
from .DisplacementBC import DisplacementBC
from .DisplacementBCState import DisplacementBCState
from .ElectricPotentialBC import ElectricPotentialBC
from .ElectricPotentialBCState import ElectricPotentialBCState
from .EulerianBC import EulerianBC
from .EulerianBCState import EulerianBCState
from .EulerianMotionBC import EulerianMotionBC
from .EulerianMotionBCState import EulerianMotionBCState
from .FluidCavityPressureBC import FluidCavityPressureBC
from .FluidCavityPressureBCState import FluidCavityPressureBCState
from .MagneticVectorPotentialBC import MagneticVectorPotentialBC
from .MaterialFlowBC import MaterialFlowBC
from .MaterialFlowBCState import MaterialFlowBCState
from .PorePressureBC import PorePressureBC
from .PorePressureBCState import PorePressureBCState
from .RetainedNodalDofsBC import RetainedNodalDofsBC
from .SecondaryBaseBC import SecondaryBaseBC
from .SecondaryBaseBCState import SecondaryBaseBCState
from .SubmodelBC import SubmodelBC
from .SubmodelBCState import SubmodelBCState
from .TemperatureBC import TemperatureBC
from .TemperatureBCState import TemperatureBCState
from .TypeBC import TypeBC
from .VelocityBaseMotionBC import VelocityBaseMotionBC
from .VelocityBaseMotionBCState import VelocityBaseMotionBCState
from .VelocityBC import VelocityBC
from .VelocityBCState import VelocityBCState


@abaqus_class_doc
class BoundaryConditionModel(ModelBase):
    """Abaqus creates a Model object named `Model-1` when a session is started.

    .. note::
        This object can be accessed by::

            mdb.models[name]
    """

    @abaqus_method_doc
    def AccelerationBaseMotionBC(
        self,
        name: str,
        createStepName: str,
        dof: Literal[C.U3, C.UR2, C.U1, C.UR3, C.UR1, C.U2],
        amplitudeScaleFactor: float = 1,
        centerOfRotation: tuple = (),
        correlation: Correlation | None = None,
        secondaryBase: str = "",
        useComplex: Boolean = OFF,
        amplitude: str = UNSET,
    ) -> AccelerationBaseMotionBC:
        """This method creates a AccelerationBaseMotionBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].AccelerationBaseMotionBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        dof
            A SymbolicConstant specifying the constrained degree-of-freedom. Possible values for the
            SymbolicConstant are U1, U2, U3, UR1, UR2, UR3. The default value is U1.
        amplitudeScaleFactor
            A Float specifying the scale factor for the amplitude curve. The default value is 1.0.
        centerOfRotation
            A ModelDot object specifying a tuple containing one center of rotation. The default
            value is the global origin. This argument applies only when **dof** = UR1, UR2, or UR3.
        correlation
            A Correlation object.
        secondaryBase
            A String specifying the name of the SecondaryBaseBC object associated with this boundary
            condition. The default value is an empty string.
        useComplex
            A Boolean specifying whether to define the imaginary (out-of-plane) portion of the base
            motion record given by amplitude definition. The default value is OFF.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.

        Returns
        -------
        bc: AccelerationBaseMotionBC
            An AccelerationBaseMotionBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = AccelerationBaseMotionBC(
            name,
            createStepName,
            dof,
            amplitudeScaleFactor,
            centerOfRotation,
            correlation,
            secondaryBase,
            useComplex,
            amplitude,
        )
        self.steps[createStepName].boundaryConditionStates[name] = AccelerationBaseMotionBCState()
        return boundaryCondition

    @abaqus_method_doc
    def AccelerationBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        fieldName: str = "",
        a1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        a2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        a3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ar1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ar2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ar3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        amplitude: str = UNSET,
        localCsys: str | None = None,
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
    ) -> AccelerationBC:
        """This method creates an AccelerationBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].AccelerationBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        fieldName
            A String specifying the name of the AnalyticalField object associated with this boundary
            condition. The **fieldName** argument applies only when **distributionType** = FIELD. The
            default value is an empty string.
        a1
            A Float or a SymbolicConstant specifying the acceleration component in the 1-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is
            UNSET. Note: Although **a1**, **a2**, **a3**, **ar1**, **ar2**, and **ar3** are optional arguments, at
            least one of them must be specified.
        a2
            A Float or a SymbolicConstant specifying the acceleration component in the 2-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        a3
            A Float or a SymbolicConstant specifying the acceleration component in the 3-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        ar1
            A Float or a SymbolicConstant specifying the rotational acceleration component about the
            1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        ar2
            A Float or a SymbolicConstant specifying the rotational acceleration component about the
            2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        ar3
            A Float or a SymbolicConstant specifying the rotational acceleration component about the
            3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.

        Returns
        -------
        bc: AccelerationBC
            An AccelerationBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = AccelerationBC(
            name,
            createStepName,
            region,
            fieldName,
            a1,
            a2,
            a3,
            ar1,
            ar2,
            ar3,
            amplitude,
            localCsys,
            distributionType,
        )
        self.steps[createStepName].boundaryConditionStates[name] = AccelerationBCState()
        return boundaryCondition

    @abaqus_method_doc
    def AcousticPressureBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        fieldName: str = "",
        magnitude: float = 0,
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
        amplitude: str = UNSET,
        fixed: Boolean = OFF,
    ) -> AcousticPressureBC:
        """This method creates a AcousticPressureBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].AcousticPressureBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        fieldName
            A String specifying the name of the AnalyticalField object associated with this boundary
            condition. The **fieldName** argument applies only when **distributionType** = FIELD. The
            default value is an empty string.
        magnitude
            A Float specifying the acoustic pressure magnitude. The default value is 0. The
            **magnitude** argument is optional if **distributionType** = USER_DEFINED.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is OFF.

        Returns
        -------
        bc: AcousticPressureBC
            An AcousticPressureBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = AcousticPressureBC(
            name,
            createStepName,
            region,
            fieldName,
            magnitude,
            distributionType,
            amplitude,
            fixed,
        )
        self.steps[createStepName].boundaryConditionStates[name] = AcousticPressureBCState()
        return boundaryCondition

    @abaqus_method_doc
    def ConcentrationBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        fieldName: str = "",
        magnitude: float = 0,
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
        amplitude: str = UNSET,
        fixed: Boolean = OFF,
    ) -> ConcentrationBC:
        """This method creates a ConcentrationBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ConcentrationBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        fieldName
            A String specifying the name of the AnalyticalField object associated with this boundary
            condition. The **fieldName** argument applies only when **distributionType** = FIELD. The
            default value is an empty string.
        magnitude
            A Float specifying the concentration magnitude. The default value is 0. The **magnitude**
            argument is optional if **distributionType** = USER_DEFINED.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is OFF.

        Returns
        -------
        bc: ConcentrationBC
            A ConcentrationBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = ConcentrationBC(
            name,
            createStepName,
            region,
            fieldName,
            magnitude,
            distributionType,
            amplitude,
            fixed,
        )
        self.steps[createStepName].boundaryConditionStates[name] = ConcentrationBCState()
        return boundaryCondition

    @abaqus_method_doc
    def ConnAccelerationBC(
        self,
        name: str,
        createStepName: str,
        region: str = "",
        fastenerName: str = "",
        fastenerSetName: str = "",
        a1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        a2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        a3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ar1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ar2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ar3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        amplitude: str = UNSET,
        distributionType: Literal[C.USER_DEFINED, C.UNIFORM] = UNIFORM,
    ) -> ConnAccelerationBC:
        """This method creates an ConnAccelerationBC object on a wire region. Alternatively, the boundary
        condition may also be applied to a wire set referenced from an assembled fastener template model.

        .. note::
            This function can be accessed by::

                mdb.models[name].ConnAccelerationBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            The wire region to which the boundary condition is applied. This argument is not valid
            when **fastenerName** and **fastenerSetName** are specified.
        fastenerName
            A String specifying the name of the assembled fastener to which the boundary condition
            will be applied. This argument is not valid when **region** is specified. When this
            argument is specified, **fastenerSetName** must also be specified. The default value is an
            empty string.
        fastenerSetName
            A String specifying the assembled fastener template model set to which the boundary
            condition will be applied. This argument is not valid when **region** is specified. When
            this argument is specified, **fastenerName** must also be specified. The default value is
            an empty string.
        a1
            A Float or a SymbolicConstant specifying the acceleration component in the connector's
            local 1-direction. Possible values for the SymbolicConstant are UNSET and SET. The
            default value is UNSET. Note: Although **a1**, **a2**, **a3**, **ar1**, **ar2**, and **ar3** are
            optional arguments, at least one of them must be specified.
        a2
            A Float or a SymbolicConstant specifying the acceleration component in the connector's
            local 2-direction. Possible values for the SymbolicConstant are UNSET and SET. The
            default value is UNSET.
        a3
            A Float or a SymbolicConstant specifying the acceleration component in the connector's
            local 3-direction. Possible values for the SymbolicConstant are UNSET and SET. The
            default value is UNSET.
        ar1
            A Float or a SymbolicConstant specifying the rotational acceleration component in the
            connector's local 4-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        ar2
            A Float or a SymbolicConstant specifying the rotational acceleration component in the
            connector's local 5-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        ar3
            A Float or a SymbolicConstant specifying the rotational acceleration component in the
            connector's local 6-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM and USER_DEFINED. The default value is UNIFORM.

        Returns
        -------
        bc: ConnAccelerationBC
            A ConnAccelerationBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = ConnAccelerationBC(
            name,
            createStepName,
            region,
            fastenerName,
            fastenerSetName,
            a1,
            a2,
            a3,
            ar1,
            ar2,
            ar3,
            amplitude,
            distributionType,
        )
        self.steps[createStepName].boundaryConditionStates[name] = ConnAccelerationBCState()
        return boundaryCondition

    @abaqus_method_doc
    def ConnDisplacementBC(
        self,
        name: str,
        createStepName: str,
        region: str = "",
        fastenerName: str = "",
        fastenerSetName: str = "",
        u1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        u2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        u3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ur1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ur2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ur3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        fixed: Boolean = OFF,
        amplitude: str = UNSET,
        distributionType: Literal[C.USER_DEFINED, C.UNIFORM] = UNIFORM,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
    ) -> ConnDisplacementBC:
        """This method creates a ConnDisplacementBC object on a wire region. Alternatively, the boundary
        condition may also be applied to a wire set referenced from an assembled fastener template model.

        .. note::
            This function can be accessed by::

                mdb.models[name].ConnDisplacementBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            The wire region to which the boundary condition is applied. This argument is not valid
            when **fastenerName** and **fastenerSetName** are specified.
        fastenerName
            A String specifying the name of the assembled fastener to which the boundary condition
            will be applied. This argument is not valid when **region** is specified. When this
            argument is specified, **fastenerSetName** must also be specified. The default value is an
            empty string.
        fastenerSetName
            A String specifying the assembled fastener template model set to which the boundary
            condition will be applied. This argument is not valid when **region** is specified. When
            this argument is specified, **fastenerName** must also be specified. The default value is
            an empty string.
        u1
            A Float, a Complex, or a SymbolicConstant specifying the displacement component in the
            connector's local 1-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET. Note: Although **u1**, **u2**, **u3**, **ur1**, **ur2**, and **ur3**
            are optional arguments, at least one of them must be specified.
        u2
            A Float, a Complex, or a SymbolicConstant specifying the displacement component in the
            connector's local 2-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        u3
            A Float, a Complex, or a SymbolicConstant specifying the displacement component in the
            connector's local 3-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        ur1
            A Float, a Complex, or a SymbolicConstant specifying the rotational component in the
            connector's local 4-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        ur2
            A Float, a Complex, or a SymbolicConstant specifying the rotational component in the
            connector's local 5-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        ur3
            A Float, a Complex, or a SymbolicConstant specifying the rotational component in the
            connector's local 6-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is OFF.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM and USER_DEFINED. The default value is UNIFORM.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.

        Returns
        -------
        bc: ConnDisplacementBC
            A ConnDisplacementBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = ConnDisplacementBC(
            name,
            createStepName,
            region,
            fastenerName,
            fastenerSetName,
            u1,
            u2,
            u3,
            ur1,
            ur2,
            ur3,
            fixed,
            amplitude,
            distributionType,
            buckleCase,
        )
        self.steps[createStepName].boundaryConditionStates[name] = ConnDisplacementBCState()
        return boundaryCondition

    @abaqus_method_doc
    def ConnVelocityBC(
        self,
        name: str,
        createStepName: str,
        region: str = "",
        fastenerName: str = "",
        fastenerSetName: str = "",
        v1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        v2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        v3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        amplitude: str = UNSET,
        distributionType: Literal[C.USER_DEFINED, C.UNIFORM] = UNIFORM,
    ) -> ConnVelocityBC:
        """This method creates a ConnVelocityBC object on a wire region. Alternatively, the boundary condition
        may also be applied to a wire set referenced from an assembled fastener template model.

        .. note::
            This function can be accessed by::

                mdb.models[name].ConnVelocityBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            The wire region to which the boundary condition is applied. This argument is not valid
            when **fastenerName** and **fastenerSetName** are specified.
        fastenerName
            A String specifying the name of the assembled fastener to which the boundary condition
            will be applied. This argument is not valid when **region** is specified. When this
            argument is specified, **fastenerSetName** must also be specified. The default value is an
            empty string.
        fastenerSetName
            A String specifying the assembled fastener template model set to which the boundary
            condition will be applied. This argument is not valid when **region** is specified. When
            this argument is specified, **fastenerName** must also be specified. The default value is
            an empty string.
        v1
            A Float or a SymbolicConstant specifying the velocity component in the connector's local
            1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET. Note: Although **v1**, **v2**, **v3**, **vr1**, **vr2**, and **vr3** are optional
            arguments, at least one of them must be specified.
        v2
            A Float or a SymbolicConstant specifying the velocity component in the connector's local
            2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        v3
            A Float or a SymbolicConstant specifying the velocity component in the connector's local
            3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        vr1
            A Float or a SymbolicConstant specifying the rotational velocity component in the
            connector's local 4-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        vr2
            A Float or a SymbolicConstant specifying the rotational velocity component in the
            connector's local 5-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        vr3
            A Float or a SymbolicConstant specifying the rotational velocity component in the
            connector's local 6-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM and USER_DEFINED. The default value is UNIFORM.

        Returns
        -------
        bc: ConnVelocityBC
            A ConnVelocityBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = ConnVelocityBC(
            name,
            createStepName,
            region,
            fastenerName,
            fastenerSetName,
            v1,
            v2,
            v3,
            vr1,
            vr2,
            vr3,
            amplitude,
            distributionType,
        )
        self.steps[createStepName].boundaryConditionStates[name] = ConnVelocityBCState()
        return boundaryCondition

    @abaqus_method_doc
    def DisplacementBaseMotionBC(
        self,
        name: str,
        createStepName: str,
        dof: Literal[C.U3, C.UR2, C.U1, C.UR3, C.UR1, C.U2],
        amplitudeScaleFactor: float = 1,
        centerOfRotation: tuple = (),
        correlation: Correlation | None = None,
        secondaryBase: str = "",
        useComplex: Boolean = OFF,
        amplitude: str = UNSET,
    ) -> DisplacementBaseMotionBC:
        """This method creates a DisplacementBaseMotionBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].DisplacementBaseMotionBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        dof
            A SymbolicConstant specifying the constrained degree-of-freedom. Possible values for the
            SymbolicConstant are U1, U2, U3, UR1, UR2, UR3. The default value is U1.
        amplitudeScaleFactor
            A Float specifying the scale factor for the amplitude curve. The default value is 1.0.
        centerOfRotation
            A ModelDot object specifying a tuple containing one center of rotation. The default
            value is the global origin. This argument applies only when **dof** = UR1, UR2, or UR3.
        correlation
            A Correlation object.
        secondaryBase
            A String specifying the name of the SecondaryBaseBC object associated with this boundary
            condition. The default value is an empty string.
        useComplex
            A Boolean specifying whether to define the imaginary (out-of-plane) portion of the base
            motion record given by amplitude definition. The default value is OFF.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.

        Returns
        -------
        bc: DisplacementBaseMotionBC
            A DisplacementBaseMotionBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = DisplacementBaseMotionBC(
            name,
            createStepName,
            dof,
            amplitudeScaleFactor,
            centerOfRotation,
            correlation,
            secondaryBase,
            useComplex,
            amplitude,
        )
        self.steps[createStepName].boundaryConditionStates[name] = DisplacementBaseMotionBCState()
        return boundaryCondition

    @abaqus_method_doc
    def DisplacementBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        fieldName: str = "",
        u1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        u2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        u3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ur1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ur2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        ur3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        fixed: Boolean = OFF,
        amplitude: str = UNSET,
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM, C.DISCRETE_FIELD] = UNIFORM,
        localCsys: str | None = None,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
    ) -> DisplacementBC:
        """This method creates a DisplacementBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].DisplacementBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        fieldName
            A String specifying the name of the AnalyticalField or DiscreteField object associated
            with this boundary condition. The **fieldName** argument applies only when
            **distributionType** = FIELD or **distributionType** = DISCRETE_FIELD. The default value is an
            empty string.
        u1
            A Float, a Complex, or a SymbolicConstant specifying the displacement component in the
            1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET. Note: Although **u1**, **u2**, **u3**, **ur1**, **ur2**, and **ur3** are optional
            arguments, at least one of them must be specified.
        u2
            A Float, a Complex, or a SymbolicConstant specifying the displacement component in the
            2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        u3
            A Float, a Complex, or a SymbolicConstant specifying the displacement component in the
            3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        ur1
            A Float, a Complex, or a SymbolicConstant specifying the rotational displacement
            component about the 1-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        ur2
            A Float, a Complex, or a SymbolicConstant specifying the rotational displacement
            component about the 2-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        ur3
            A Float, a Complex, or a SymbolicConstant specifying the rotational displacement
            component about the 3-direction. Possible values for the SymbolicConstant are UNSET and
            SET. The default value is UNSET.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is OFF.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, FIELD, and DISCRETE_FIELD. The default value
            is UNIFORM.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.

        Returns
        -------
        bc: DisplacementBC
            A DisplacementBC object
        """
        self.boundaryConditions[name] = boundaryCondition = DisplacementBC(
            name,
            createStepName,
            region,
            fieldName,
            u1,
            u2,
            u3,
            ur1,
            ur2,
            ur3,
            fixed,
            amplitude,
            distributionType,
            localCsys,
            buckleCase,
        )
        self.steps[createStepName].boundaryConditionStates[name] = DisplacementBCState()
        return boundaryCondition

    @abaqus_method_doc
    def ElectricPotentialBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        fieldName: str = "",
        magnitude: float = 0,
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
        amplitude: str = UNSET,
        fixed: Boolean = OFF,
    ) -> ElectricPotentialBC:
        """This method creates an ElectricPotentialBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].ElectricPotentialBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        fieldName
            A String specifying the name of the AnalyticalField object associated with this boundary
            condition. The **fieldName** argument applies only when **distributionType** = FIELD. The
            default value is an empty string.
        magnitude
            A Float specifying the electrical potential magnitude. The default value is 0. The
            **magnitude** argument is optional if **distributionType** = USER_DEFINED.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is OFF.

        Returns
        -------
        bc: ElectricPotentialBC
            An ElectricPotentialBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = ElectricPotentialBC(
            name,
            createStepName,
            region,
            fieldName,
            magnitude,
            distributionType,
            amplitude,
            fixed,
        )
        self.steps[createStepName].boundaryConditionStates[name] = ElectricPotentialBCState()
        return boundaryCondition

    @abaqus_method_doc
    def EulerianBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        definition: Literal[C.BOTH, C.INFLOW, C.OUTFLOW] = INFLOW,
        inflowType: Literal[C.VOID, C.FREE, C.NONE] = FREE,
        outflowType: Literal[C.FREE, C.ZERO_PRESSURE, C.EQUILIBRIUM, C.NON_REFLECTING] = ZERO_PRESSURE,
    ) -> EulerianBC:
        """This method creates a EulerianBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].EulerianBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        definition
            A SymbolicConstant specifying the flow conditions to be defined. Possible values are
            INFLOW, OUTFLOW, and BOTH. The default value is INFLOW.
        inflowType
            A SymbolicConstant specifying the control of material flow into the Eulerian domain.
            Possible values are FREE, NONE, and VOID. The default value is FREE.
        outflowType
            A SymbolicConstant specifying the control of flow of material out of the Eulerian
            domain. Possible values are ZERO_PRESSURE, FREE, NON_REFLECTING, and EQUILIBRIUM. The
            default value is ZERO_PRESSURE.

        Returns
        -------
        bc: EulerianBC
            An EulerianBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = EulerianBC(
            name, createStepName, region, definition, inflowType, outflowType
        )
        self.steps[createStepName].boundaryConditionStates[name] = EulerianBCState()
        return boundaryCondition

    @abaqus_method_doc
    def EulerianMotionBC(
        self,
        name: str,
        createStepName: str,
        instanceName: str,
        followRegion: Boolean = ON,
        region: Region | None = None,
        materialName: str = "",
        ctrPosition1: Literal[C.FREE, C.FIXED] = FREE,
        posPosition1: Literal[C.FREE, C.FIXED] = FREE,
        negPosition1: Literal[C.FREE, C.FIXED] = FREE,
        expansionRatio1: float | None = None,
        contractRatio1: float = 0,
        ctrPosition2: Literal[C.FREE, C.FIXED] = FREE,
        posPosition2: Literal[C.FREE, C.FIXED] = FREE,
        negPosition2: Literal[C.FREE, C.FIXED] = FREE,
        expansionRatio2: float | None = None,
        contractRatio2: float = 0,
        ctrPosition3: Literal[C.FREE, C.FIXED] = FREE,
        posPosition3: Literal[C.FREE, C.FIXED] = FREE,
        negPosition3: Literal[C.FREE, C.FIXED] = FREE,
        expansionRatio3: float | None = None,
        contractRatio3: float = 0,
        allowContraction: Boolean = ON,
        aspectLimit: float = 10,
        vmaxFactor: float = 1,
        volThreshold: float = 0,
        bufferSize: float = 2,
    ) -> EulerianMotionBC:
        """This method creates an EulerianMotionBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].EulerianMotionBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        instanceName
            A String specifying the name of the Eulerian part instance.
        followRegion
            A Boolean specifying whether the mesh will follow a regular surface region or an
            Eulerian surface. The default value is ON.
        region
            A Region object specifying the region to which the boundary condition is applied.
        materialName
            A String specifying the name of the Eulerian surface to follow. This argument applies
            only when **followRegion** = False.
        ctrPosition1
            A SymbolicConstant specifying the 1-direction translational constraint on the center of
            the Eulerian mesh. Possible values are FREE and FIXED. The default value is FREE.
        posPosition1
            A SymbolicConstant specifying the translational constraint on the positive (maximum)
            bounds of the mesh in the 1 direction. Possible values are FREE and FIXED. The default
            value is FREE.
        negPosition1
            A SymbolicConstant specifying the translational constraint on the negative (minimum)
            bounds of the mesh in the 1 direction. Possible values are FREE and FIXED. The default
            value is FREE.
        expansionRatio1
            None or a Float specifying the upper bounds on the allowable scaling of the mesh in the
            1 direction. If **expansionRatio1** = None, then there is no upper limit. The default value
            is None.
        contractRatio1
            A Float specifying the lower bounds on the allowable scaling of the mesh in the 1
            direction. The default value is 0.0.
        ctrPosition2
            A SymbolicConstant specifying the 2-direction translational constraint on the center of
            the Eulerian mesh. Possible values are FREE and FIXED. The default value is FREE.
        posPosition2
            A SymbolicConstant specifying the translational constraint on the positive (maximum)
            bounds of the mesh in the 2 direction. Possible values are FREE and FIXED. The default
            value is FREE.
        negPosition2
            A SymbolicConstant specifying the translational constraint on the negative (minimum)
            bounds of the mesh in the 2 direction. Possible values are FREE and FIXED. The default
            value is FREE.
        expansionRatio2
            None or a Float specifying the upper bounds on the allowable scaling of the mesh in the
            2 direction. If **expansionRatio2** = None, then there is no upper limit. The default value
            is None.
        contractRatio2
            A Float specifying the lower bounds on the allowable scaling of the mesh in the 2
            direction. The default value is 0.0.
        ctrPosition3
            A SymbolicConstant specifying the 3-direction translational constraint on the center of
            the Eulerian mesh. Possible values are FREE and FIXED. The default value is FREE.
        posPosition3
            A SymbolicConstant specifying the translational constraint on the positive (maximum)
            bounds of the mesh in the 3 direction. Possible values are FREE and FIXED. The default
            value is FREE.
        negPosition3
            A SymbolicConstant specifying the translational constraint on the negative (minimum)
            bounds of the mesh in the 3 direction. Possible values are FREE and FIXED. The default
            value is FREE.
        expansionRatio3
            None or a Float specifying the upper bounds on the allowable scaling of the mesh in the
            3 direction. If **expansionRatio3** = None, then there is no upper limit. The default value
            is None.
        contractRatio3
            A Float specifying the lower bounds on the allowable scaling of the mesh in the 3
            direction. The default value is 0.0.
        allowContraction
            A Boolean specifying whether the mesh is allowed to contract . The default value is ON.
        aspectLimit
            A Float specifying the maximum change in allowed aspect ratio (for any of the three mesh
            aspects, 1-2, 2-3, 3-1). The default value is 10.0.
        vmaxFactor
            A Float specifying the multiplier for the mesh nodal velocity limit. The default value
            is 1.01.
        volThreshold
            A Float specifying the lower bounds on the volume fraction when determining which nodes
            to include in the surface bounding box calculation for an Eulerian material surface.
            This argument applies only when **followRegion** = False. The default value is 0.5.
        bufferSize
            None or a Float specifying the buffer between the surface box and the Eulerian section
            mesh bounding box. The default value is 2.0.

        Returns
        -------
        bc: EulerianMotionBC
            An EulerianMotionBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = EulerianMotionBC(
            name,
            createStepName,
            instanceName,
            followRegion,
            region,
            materialName,
            ctrPosition1,
            posPosition1,
            negPosition1,
            expansionRatio1,
            contractRatio1,
            ctrPosition2,
            posPosition2,
            negPosition2,
            expansionRatio2,
            contractRatio2,
            ctrPosition3,
            posPosition3,
            negPosition3,
            expansionRatio3,
            contractRatio3,
            allowContraction,
            aspectLimit,
            vmaxFactor,
            volThreshold,
            bufferSize,
        )
        self.steps[createStepName].boundaryConditionStates[name] = EulerianMotionBCState()
        return boundaryCondition

    @abaqus_method_doc
    def FluidCavityPressureBC(
        self,
        name: str,
        createStepName: str,
        fluidCavity: str,
        magnitude: float = 0,
        amplitude: str = UNSET,
        fixed: Boolean = OFF,
    ) -> FluidCavityPressureBC:
        """This method creates a FluidCavityPressureBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].FluidCavityPressureBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        fluidCavity
            A String specifying the name of a Fluid Cavity Interaction.
        magnitude
            A Float specifying the fluid cavity pressure magnitude. The default value is 0.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is OFF.

        Returns
        -------
        bc: FluidCavityPressureBC
            A FluidCavityPressureBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = FluidCavityPressureBC(
            name, createStepName, fluidCavity, magnitude, amplitude, fixed
        )
        self.steps[createStepName].boundaryConditionStates[name] = FluidCavityPressureBCState()
        return boundaryCondition

    @abaqus_method_doc
    def MagneticVectorPotentialBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        component1: Literal[C.SET, C.UNSET] | None = None,
        component2: Literal[C.SET, C.UNSET] = UNSET,
        component3: Literal[C.SET, C.UNSET] = UNSET,
        amplitude: str = UNSET,
        distributionType: Literal[C.USER_DEFINED, C.UNIFORM] = UNIFORM,
        localCsys: str | None = None,
    ) -> MagneticVectorPotentialBC:
        """This method creates a MagneticVectorPotentialBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].MagneticVectorPotentialBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        component1
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET
        component2
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        component3
            A Complex, or a SymbolicConstant specifying the magnetic vector potential component in
            the 3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM and USER_DEFINED. The default value is UNIFORM.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        bc: MagneticVectorPotentialBC
            A MagneticVectorPotentialBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = MagneticVectorPotentialBC(
            name,
            createStepName,
            region,
            component1,
            component2,
            component3,
            amplitude,
            distributionType,
            localCsys,
        )
        return boundaryCondition

    @abaqus_method_doc
    def MaterialFlowBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        fieldName: str = "",
        magnitude: float = 0,
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
        amplitude: str = UNSET,
        fixed: Boolean = OFF,
    ) -> MaterialFlowBC:
        """This method creates a MaterialFlowBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].MaterialFlowBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        fieldName
            A String specifying the name of the AnalyticalField object associated with this boundary
            condition. The **fieldName** argument applies only when **distributionType** = FIELD. The
            default value is an empty string.
        magnitude
            A Float specifying the material flow magnitude. The default value is 0. The **magnitude**
            argument is optional if **distributionType** = USER_DEFINED.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is OFF.

        Returns
        -------
        bc: MaterialFlowBC
            A MaterialFlowBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = MaterialFlowBC(
            name,
            createStepName,
            region,
            fieldName,
            magnitude,
            distributionType,
            amplitude,
            fixed,
        )
        self.steps[createStepName].boundaryConditionStates[name] = MaterialFlowBCState()
        return boundaryCondition

    @abaqus_method_doc
    def PorePressureBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        fieldName: str = "",
        magnitude: float = 0,
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
        amplitude: str = UNSET,
        fixed: Boolean = OFF,
    ) -> PorePressureBC:
        """This method creates a PorePressureBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].PorePressureBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        fieldName
            A String specifying the name of the AnalyticalField object associated with this boundary
            condition. The **fieldName** argument applies only when **distributionType** = FIELD. The
            default value is an empty string.
        magnitude
            A Float specifying the pore pressure magnitude. The default value is 0. The **magnitude**
            argument is optional if **distributionType** = USER_DEFINED.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is OFF.

        Returns
        -------
        bc: PorePressureBC
            A PorePressureBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = PorePressureBC(
            name,
            createStepName,
            region,
            fieldName,
            magnitude,
            distributionType,
            amplitude,
            fixed,
        )
        self.steps[createStepName].boundaryConditionStates[name] = PorePressureBCState()
        return boundaryCondition

    @abaqus_method_doc
    def RetainedNodalDofsBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        u1: Boolean = OFF,
        u2: Boolean = OFF,
        u3: Boolean = OFF,
        ur1: Boolean = OFF,
        ur2: Boolean = OFF,
        ur3: Boolean = OFF,
    ) -> RetainedNodalDofsBC:
        """This method creates a RetainedNodalDofsBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].RetainedNodalDofsBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        u1
            A Boolean specifying whether to retain the degree of freedom in the 1-direction. The
            default value is OFF indicating that the degree of freedom is not retained.
        u2
            A Boolean specifying whether to retain the degree of freedom in the 2-direction. The
            default value is OFF indicating that the degree of freedom is not retained.
        u3
            A Boolean specifying whether to retain the degree of freedom in the 3-direction. The
            default value is OFF indicating that the degree of freedom is not retained.
        ur1
            A Boolean specifying whether to retain the rotational degree of freedom about the
            1-direction. The default value is OFF indicating that the degree of freedom is not
            retained.
        ur2
            A Boolean specifying whether to retain the rotational degree of freedom about the
            2-direction. The default value is OFF indicating that the degree of freedom is not
            retained.
        ur3
            A Boolean specifying whether to retain the rotational degree of freedom about the
            3-direction. The default value is OFF indicating that the degree of freedom is not
            retained.

        Returns
        -------
        bc: RetainedNodalDofsBC
            A RetainedNodalDofsBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = RetainedNodalDofsBC(
            name, createStepName, region, u1, u2, u3, ur1, ur2, ur3
        )
        return boundaryCondition

    @abaqus_method_doc
    def SecondaryBaseBC(self, name: str, createStepName: str, regions: RegionArray, dofs: tuple) -> SecondaryBaseBC:
        """This method creates a SecondaryBaseBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].SecondaryBaseBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        regions
            A RegionArray object specifying the region to which the boundary condition is applied.
            Note that the usual **region** is ignored. The default value is MODEL.
        dofs
            A sequence of sequences of Ints specifying the constrained degrees-of-freedom.

        Returns
        -------
        bc: SecondaryBaseBC
            A SecondaryBaseBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = SecondaryBaseBC(name, createStepName, regions, dofs)
        self.steps[createStepName].boundaryConditionStates[name] = SecondaryBaseBCState()
        return boundaryCondition

    @abaqus_method_doc
    def SubmodelBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        dof: tuple,
        globalStep: str,
        timeScale: Boolean,
        shellThickness: float,
        globalDrivingRegion: str = "",
        absoluteExteriorTolerance: float | None = None,
        exteriorTolerance: float = 0,
        localCsys: str | None = None,
        globalIncrement: int = 0,
        centerZoneSize: float | None = None,
    ) -> SubmodelBC:
        """This method creates a SubmodelBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].SubmodelBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        dof
            A sequence of Ints specifying the degrees of freedom to which the boundary condition is
            applied.
        globalStep
            A String specifying the step in the global model from which Abaqus reads the values of
            the variables that will drive the submodel analysis. The String indicates the position
            of the step in the sequence of analysis steps. For example, **globalStep** = '1' indicates
            the first step.
        timeScale
            A Boolean specifying whether to scale the time variable for the driven nodes' amplitude
            functions to match the submodel analysis step time. The default value is OFF.
        shellThickness
            A Float specifying the thickness of the shell in the global model. This argument is
            required for shell-to-solid submodeling and is not applicable to other submodels. The
            default value is 0.0.
        globalDrivingRegion
            A String specifying the element set in the global model that will be searched for
            elements whose responses will be used to drive the submodel. An empty string indicates
            that the entire global model will be searched. The default value is an empty string.
        absoluteExteriorTolerance
            None or a Float specifying the absolute value by which a driven node of the submodel can
            lie outside the region of the elements of the global model. The default value is None.
        exteriorTolerance
            None or a Float specifying the fraction of the average element size in the global model
            by which a driven node of the submodel can lie outside the region of the elements of the
            global model. The default value is 0.05.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        globalIncrement
            An Int specifying the increment number in the global model step from which the solution
            will be used to specify the values of the driven variables. If **globalIncrement** = 0, the
            solution from the last increment will be used. The **globalIncrement** argument is
            applicable only for linear perturbation steps. The default value is 0.
        centerZoneSize
            A Float specifying the thickness of the center zone size around the shell midsurface.
            The default value is None.

        Returns
        -------
        bc: SubmodelBC
            A SubmodelBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = SubmodelBC(
            name,
            createStepName,
            region,
            dof,
            globalStep,
            timeScale,
            shellThickness,
            globalDrivingRegion,
            absoluteExteriorTolerance,
            exteriorTolerance,
            localCsys,
            globalIncrement,
            centerZoneSize,
        )
        self.steps[createStepName].boundaryConditionStates[name] = SubmodelBCState()
        return boundaryCondition

    @abaqus_method_doc
    def TemperatureBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        fieldName: str = "",
        magnitude: float = 0,
        dof: tuple = (),
        amplitude: str = UNSET,
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
        fixed: Boolean = OFF,
    ) -> TemperatureBC:
        """This method creates a TemperatureBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].TemperatureBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        fieldName
            A String specifying the name of the AnalyticalField object associated with this boundary
            condition. The **fieldName** argument applies only when **distributionType** = FIELD. The
            default value is an empty string.
        magnitude
            A Float specifying the temperature magnitude. The default value is 0.
        dof
            A sequence of Ints specifying the degrees of freedom to which the boundary condition is
            applied. The default value is (11,).
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.
        fixed
            A Boolean specifying whether the boundary condition should remain fixed at the current
            values at the start of the step. The default value is OFF.

        Returns
        -------
        bc: TemperatureBC
            A TemperatureBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = TemperatureBC(
            name,
            createStepName,
            region,
            fieldName,
            magnitude,
            dof,
            amplitude,
            distributionType,
            fixed,
        )
        self.steps[createStepName].boundaryConditionStates[name] = TemperatureBCState()
        return boundaryCondition

    @abaqus_method_doc
    def VelocityBaseMotionBC(
        self,
        name: str,
        createStepName: str,
        dof: Literal[C.U3, C.UR2, C.U1, C.UR3, C.UR1, C.U2],
        amplitudeScaleFactor: float = 1,
        centerOfRotation: tuple = (),
        correlation: Correlation | None = None,
        secondaryBase: str = "",
        useComplex: Boolean = OFF,
        amplitude: str = UNSET,
    ) -> VelocityBaseMotionBC:
        """This method creates a VelocityBaseMotionBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].VelocityBaseMotionBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        dof
            A SymbolicConstant specifying the constrained degree-of-freedom. Possible values for the
            SymbolicConstant are U1, U2, U3, UR1, UR2, UR3. The default value is U1.
        amplitudeScaleFactor
            A Float specifying the scale factor for the amplitude curve. The default value is 1.0.
        centerOfRotation
            A ModelDot object specifying a tuple containing one center of rotation. The default
            value is the global origin. This argument applies only when **dof** = UR1, UR2, or UR3.
        correlation
            A Correlation object.
        secondaryBase
            A String specifying the name of the SecondaryBaseBC object associated with this boundary
            condition. The default value is an empty string.
        useComplex
            A Boolean specifying whether to define the imaginary (out-of-plane) portion of the base
            motion record given by amplitude definition. The default value is OFF.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.

        Returns
        -------
        bc: VelocityBaseMotionBC
            A VelocityBaseMotionBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = VelocityBaseMotionBC(
            name,
            createStepName,
            dof,
            amplitudeScaleFactor,
            centerOfRotation,
            correlation,
            secondaryBase,
            useComplex,
            amplitude,
        )
        self.steps[createStepName].boundaryConditionStates[name] = VelocityBaseMotionBCState()
        return boundaryCondition

    @abaqus_method_doc
    def VelocityBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        fieldName: str = "",
        v1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        v2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        v3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr1: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr2: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        vr3: Union[Literal[C.SET, C.UNSET], float] = UNSET,
        amplitude: str = UNSET,
        localCsys: str | None = None,
        distributionType: Literal[C.USER_DEFINED, C.FIELD, C.UNIFORM] = UNIFORM,
    ) -> VelocityBC:
        """This method creates a VelocityBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].VelocityBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        fieldName
            A String specifying the name of the AnalyticalField object associated with this boundary
            condition. The **fieldName** argument applies only when **distributionType** = FIELD. The
            default value is an empty string.
        v1
            A Float or a SymbolicConstant specifying the velocity component in the 1-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is
            UNSET. Note: Although **v1**, **v2**, **v3**, **vr1**, **vr2**, and **vr3** are optional arguments, at
            least one of them must be specified.
        v2
            A Float or a SymbolicConstant specifying the velocity component in the 2-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        v3
            A Float or a SymbolicConstant specifying the velocity component in the 3-direction.
            Possible values for the SymbolicConstant are UNSET and SET. The default value is UNSET.
        vr1
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            1-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        vr2
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            2-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        vr3
            A Float or a SymbolicConstant specifying the rotational velocity component about the
            3-direction. Possible values for the SymbolicConstant are UNSET and SET. The default
            value is UNSET.
        amplitude
            A String or the SymbolicConstant UNSET specifying the name of the amplitude reference.
            UNSET should be used if the boundary condition has no amplitude reference. The default
            value is UNSET. You should provide the **amplitude** argument only if it is valid for the
            specified step.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.
        distributionType
            A SymbolicConstant specifying how the boundary condition is distributed spatially.
            Possible values are UNIFORM, USER_DEFINED, and FIELD. The default value is UNIFORM.

        Returns
        -------
        bc: VelocityBC
            A VelocityBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = VelocityBC(
            name,
            createStepName,
            region,
            fieldName,
            v1,
            v2,
            v3,
            vr1,
            vr2,
            vr3,
            amplitude,
            localCsys,
            distributionType,
        )
        self.steps[createStepName].boundaryConditionStates[name] = VelocityBCState()
        return boundaryCondition

    @abaqus_method_doc
    def EncastreBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
        localCsys: str | None = None,
    ) -> TypeBC:
        """This method creates an encastre TypeBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].EncastreBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = TypeBC.EncastreBC(
            name,
            createStepName,
            region,
            buckleCase,
            localCsys,
        )
        return boundaryCondition

    @abaqus_method_doc
    def PinnedBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
        localCsys: str | None = None,
    ) -> TypeBC:
        """This method creates a pinned TypeBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].PinnedBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = TypeBC.PinnedBC(
            name,
            createStepName,
            region,
            buckleCase,
            localCsys,
        )
        return boundaryCondition

    @abaqus_method_doc
    def XsymmBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
        localCsys: str | None = None,
    ) -> TypeBC:
        """This method creates a TypeBC object that specifies symmetry about the **X** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].EncastreBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = TypeBC.XsymmBC(
            name,
            createStepName,
            region,
            buckleCase,
            localCsys,
        )
        return boundaryCondition

    @abaqus_method_doc
    def YsymmBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
        localCsys: str | None = None,
    ) -> TypeBC:
        """This method creates a TypeBC object that specifies symmetry about the **Y** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].YsymmBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = TypeBC.YsymmBC(
            name,
            createStepName,
            region,
            buckleCase,
            localCsys,
        )
        return boundaryCondition

    @abaqus_method_doc
    def ZsymmBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
        localCsys: str | None = None,
    ) -> TypeBC:
        """This method creates a TypeBC object that specifies symmetry about the **Z** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].EncastreBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = TypeBC.ZsymmBC(
            name,
            createStepName,
            region,
            buckleCase,
            localCsys,
        )
        return boundaryCondition

    @abaqus_method_doc
    def XasymmBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
        localCsys: str | None = None,
    ) -> TypeBC:
        """This method creates a TypeBC object that specifies antisymmetry about the **X** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].EncastreBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = TypeBC.XasymmBC(
            name,
            createStepName,
            region,
            buckleCase,
            localCsys,
        )
        return boundaryCondition

    @abaqus_method_doc
    def YasymmBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
        localCsys: str | None = None,
    ) -> TypeBC:
        """This method creates a TypeBC object that specifies antisymmetry about the **Y** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].YasymmBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = TypeBC.YasymmBC(
            name,
            createStepName,
            region,
            buckleCase,
            localCsys,
        )
        return boundaryCondition

    @abaqus_method_doc
    def ZasymmBC(
        self,
        name: str,
        createStepName: str,
        region: Region,
        buckleCase: Literal[
            C.STRESS_PERTURBATION, C.NOT_APPLICABLE, C.BUCKLING_MODES, C.BUCKLE, C.PERTURBATION_AND_BUCKLING
        ] = NOT_APPLICABLE,
        localCsys: str | None = None,
    ) -> TypeBC:
        """This method creates a TypeBC object that specifies antisymmetry about the **Z** axis.

        .. note::
            This function can be accessed by::

                mdb.models[name].ZasymmBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        region
            A Region object specifying the region to which the boundary condition is applied.
        buckleCase
            A SymbolicConstant specifying how the boundary condition is defined in a BUCKLE
            analysis. Possible values are NOT_APPLICABLE, STRESS_PERTURBATION, BUCKLING_MODES, and
            PERTURBATION_AND_BUCKLING. The default value is NOT_APPLICABLE.
        localCsys
            None or a DatumCsys object specifying the local coordinate system of the boundary
            condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
            in the global coordinate system. The default value is None.

        Returns
        -------
        TypeBC
            A TypeBC object.
        """
        self.boundaryConditions[name] = boundaryCondition = TypeBC.ZasymmBC(
            name,
            createStepName,
            region,
            buckleCase,
            localCsys,
        )
        return boundaryCondition



from __future__ import annotations

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..Region.Region import Region
from ..Region.RegionArray import RegionArray
from ..UtilityAndView.abaqusConstants import MODEL, SymbolicConstant
from .BoundaryCondition import BoundaryCondition


@abaqus_class_doc
class SecondaryBaseBC(BoundaryCondition):
    """The SecondaryBaseBC object stores the data for a slave base boundary condition. The SecondaryBaseBC
    object is derived from the BoundaryCondition object.

    .. note::
        This object can be accessed by::

            import load
            mdb.models[name].boundaryConditions[name]
    """

    #: A String specifying the boundary condition repository key.
    name: str = ""

    #: A tuple of tuples of Ints specifying the constrained degrees-of-freedom.
    dofs: int | None = None

    #: A RegionArray object specifying the region to which the boundary condition is applied.
    #: Note that the usual **region** is ignored. The default value is MODEL.
    regions: RegionArray = MODEL

    #: A SymbolicConstant specifying the category of the boundary condition. Possible values
    #: are MECHANICAL and THERMAL.
    category: SymbolicConstant

    #: A Region object specifying the region to which the boundary condition is applied.
    region: Region = Region()

    #: None or a DatumCsys object specifying the local coordinate system of the boundary
    #: condition's degrees of freedom. If **localCsys** = None, the degrees of freedom are defined
    #: in the global coordinate system. The default value is None.
    localCsys: str | None = None

    @abaqus_method_doc
    def __init__(self, name: str, createStepName: str, regions: RegionArray, dofs: tuple):
        """This method creates a SecondaryBaseBC object.

        .. note::
            This function can be accessed by::

                mdb.models[name].SecondaryBaseBC

        Parameters
        ----------
        name
            A String specifying the boundary condition repository key.
        createStepName
            A String specifying the name of the step in which the boundary condition is created.
        regions
            A RegionArray object specifying the region to which the boundary condition is applied.
            Note that the usual **region** is ignored. The default value is MODEL.
        dofs
            A sequence of sequences of Ints specifying the constrained degrees-of-freedom.

        Returns
        -------
        SecondaryBaseBC
            A SecondaryBaseBC object.
        """
        super().__init__()

    @abaqus_method_doc
    def setValues(self, *args, **kwargs):
        """This method modifies the data for an existing SecondaryBaseBC object in the step where it is
        created."""
        ...

    @abaqus_method_doc
    def setValuesInStep(self, stepName: str):
        """This method modifies the propagating data for an existing SecondaryBaseBC object in the specified
        step.

        Parameters
        ----------
        stepName
            A String specifying the name of the step in which the boundary condition is modified.
        """
        ...



from __future__ import annotations

from .AFXTarget import AFXTarget


class AFXIntTarget(AFXTarget):
    """This class is designed for integer targets."""

    def __init__(self, initialValue: int = 0):
        """Constructor.

        Parameters
        ----------
        initialValue : int
            Initial value.
        """

    def getTypeName(self):
        """Returns the name of the target type ("Int").

        Implements AFXTarget.
        """

    def getValue(self):
        """Returns the target's current value."""

    def setValue(self, newValue: int):
        """Sets the target's current value.

        Parameters
        ----------
        newValue : int
            New value.
        """



from __future__ import annotations

from .AFXGuiObjectManager import AFXGuiObjectManager
from .FXComposite import FXComposite
from .FXIcon import FXIcon
from .FXMenuCommand import FXMenuCommand
from .FXObject import FXObject


class AFXMenuCommand(FXMenuCommand):
    """This class provides the interface for creating an FXMenuCommand and performing various management
    activities on it.

    It will use utility methods so the menu command is correctly managed for modules and toolsets.
    """

    def __init__(
        self,
        owner: AFXGuiObjectManager,
        p: FXComposite,
        label: str,
        ic: FXIcon | None = None,
        tgt: FXObject | None = None,
        sel: int = 0,
    ):
        """Constructor.

        Parameters
        ----------
        owner : AFXGuiObjectManager
            Creator of the menu command.
        p : FXComposite
            Parent widget.
        label : str
            Label for the menu button.
        ic : FXIcon | None
            Menu button icon.
        tgt : FXObject | None
            Message target.
        sel : int
            Message ID.
        """

    def getOwner(self):
        """Returns the owner of the menu command.

        Reimplemented from FXWindow.
        """



from __future__ import annotations

from typing import List

from .DirectDampingByFrequencyComponent import DirectDampingByFrequencyComponent

DirectDampingByFrequencyComponentArray = List[DirectDampingByFrequencyComponent]



from __future__ import annotations

from .constants import DEFAULT_SPACING
from .FXComposite import FXComposite
from .FXPacker import FXPacker

#: Title is left-justified.
GROUPBOX_TITLE_LEFT: int = hash("GROUPBOX_TITLE_LEFT")

#: Title is centered.
GROUPBOX_TITLE_CENTER: int = hash("GROUPBOX_TITLE_CENTER")

#: Title is right-justified.
GROUPBOX_TITLE_RIGHT: int = hash("GROUPBOX_TITLE_RIGHT")

#: Normal group box style.
GROUPBOX_NORMAL: int = hash("GROUPBOX_NORMAL")


class FXGroupBox(FXPacker):
    """A group box widget provides a nice raised or sunken border around a group of widgets, providing a visual
    delineation.

    Typically, a title is placed over the border to provide some clarification. Radio buttons placed inside
    a group box automatically assume mutually exclusive behaviour, i.e. at most one radio button will be
    checked at any one time.
    """

    def __init__(
        self,
        p: FXComposite,
        text: str,
        opts: int = GROUPBOX_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = DEFAULT_SPACING,
        pr: int = DEFAULT_SPACING,
        pt: int = DEFAULT_SPACING,
        pb: int = DEFAULT_SPACING,
        hs: int = DEFAULT_SPACING,
        vs: int = DEFAULT_SPACING,
    ):
        """Construct group box layout manager.

        Parameters
        ----------
        p : FXComposite

        text : str

        opts : int

        x : int

        y : int

        w : int

        h : int

        pl : int

        pr : int

        pt : int

        pb : int

        hs : int

        vs : int
        """

    def create(self):
        """Create server-side resources.

        Reimplemented from FXComposite.
        """

    def detach(self):
        """Detach server-side resources.

        Reimplemented from FXComposite.
        """

    def disable(self):
        """Disable the window.

        Reimplemented from FXWindow.
        """

    def enable(self):
        """Enable the window.

        Reimplemented from FXWindow.
        """

    def getDefaultHeight(self):
        """Return default height.

        Reimplemented from FXPacker.
        """

    def getDefaultWidth(self):
        """Return default width.

        Reimplemented from FXPacker.
        """

    def getText(self):
        """Return current groupbox title text."""

    def setText(self, text: str):
        """Change group box title text.

        Parameters
        ----------
        text : str
        """



import os
import sys
import warnings

from .cli import abaqus
from .config import config


def run(cae: bool = True) -> None:
    """Runs Abaqus command in system's CLI.

    This function uses the top level script file to run the Abaqus
    command, and the arguments passed to it

    Parameters
    ----------
    cae : bool, optional
        Whether to use `abaqus cae` command, True for `abaqus cae`, False for `abaqus python`, by default True
    """
    # If making docs or skipping abaqus, do nothing
    if config.make_docs or config.skip_abaqus:
        return

    # If it is a jupyter notebook, convert it to python script
    try:  # If it is a jupyter notebook
        import ipynbname

        filePath = ipynbname.path()
        print("You are running a jupyter notebook, it will be converted to a pure python script.")
        os.system(f"jupyter nbconvert --to python {filePath}")
        filePath = os.path.relpath(filePath.with_suffix(".py"))
    except (FileNotFoundError, ImportError, Exception):
        # Get the main script file
        main = sys.modules["__main__"]
        if not hasattr(main, "__file__") or main.__file__ is None:
            raise RuntimeError("Cannot find the main script file, please run the script in a file.")
        filePath = os.path.relpath(main.__file__)

    # Alternative to use abaqus command line options at run time
    print("The script will be submitted to Abaqus next and the current Python session will be closed.")
    gettrace = getattr(sys, "gettrace", None)
    if config.debug or (gettrace is not None and gettrace()):
        warnings.warn(
            "You are running the script in debug mode, the script will be opened in Abaqus PDE where you can debug it."
        )
        abaqus.pde(script=filePath)
    elif cae:
        abaqus.cae(filePath, *sys.argv[1:], **config.cae.model_dump())
    else:
        abaqus.python(filePath, *sys.argv[1:], **config.python.model_dump())
    sys.exit(0)



from __future__ import annotations

from .AFXDataDialog import AFXDataDialog
from .AFXProcedure import AFXProcedure
from .AFXStep import AFXStep


class AFXDialogStep(AFXStep):
    """This class provides dialog steps in GUI procedures."""

    def __init__(self, owner: AFXProcedure, dialog: AFXDataDialog):
        """Constructor that supplies a default prompt for the prompt area.

        Parameters
        ----------
        owner : AFXProcedure
            Procedure creating the step.
        dialog : AFXDataDialog
            Dialog box to be posted in this step.
        """

    def onCancel(self):
        """Called when the step is cancelled.

        Reimplemented from AFXStep.
        """

    def onExecute(self):
        """Called to execute steps returned by getFirstStep and getNextStep.

        Reimplemented from AFXStep.
        """

    def onResume(self):
        """Called when the step is resumed.

        Reimplemented from AFXStep.
        """

    def onSuspend(self):
        """Called when the step is suspended.

        Reimplemented from AFXStep. By clicking on Send, you accept that Dassault Systèmes will process your
        personal data and may contact you for further information. [Privacy Policy](
        https://www.3ds.com/privacy-policy).
        Total Results: Results per page This page cannot be found. The page might not exist or is temporarily unavailable. Try again or try searching for the topic.
        """



from __future__ import annotations

from .AFXBaseTable import AFXBaseTable
from .AFXDataComponent import AFXDataComponent
from .constants import DEFAULT_MARGIN
from .FXColor import FXColor
from .FXComposite import FXComposite
from .FXFont import FXFont
from .FXIcon import FXIcon
from .FXObject import FXObject

#: Allow users to resize columns.
AFXTABLE_COLUMN_RESIZABLE: int = hash("AFXTABLE_COLUMN_RESIZABLE")

#: Allow users to resize rows.
AFXTABLE_ROW_RESIZABLE: int = hash("AFXTABLE_ROW_RESIZABLE")

#: Allow users to resize rows and columns.
AFXTABLE_RESIZE: int = hash("AFXTABLE_RESIZE")

#: Disallow column selections (selecting a header/footer item in a column selects the whole column).
AFXTABLE_NO_COLUMN_SELECT: int = hash("AFXTABLE_NO_COLUMN_SELECT")

#: Disallow row selections (selecting a header/footer item in a row selects the whole row).
AFXTABLE_NO_ROW_SELECT: int = hash("AFXTABLE_NO_ROW_SELECT")

#: Selecting any item in a row selects the whole row.
AFXTABLE_ROW_MODE: int = hash("AFXTABLE_ROW_MODE")

#: Use extended selection mode that allows multiple items to be selected and allows users to drag-select a range of items.
AFXTABLE_EXTENDED_SELECT: int = hash("AFXTABLE_EXTENDED_SELECT")

#: Use single selection mode that allows up to one item to be selected.
AFXTABLE_SINGLE_SELECT: int = hash("AFXTABLE_SINGLE_SELECT")

#: Use browse selection mode that enforces one single item to be selected at all times.
AFXTABLE_BROWSE_SELECT: int = hash("AFXTABLE_BROWSE_SELECT")

#: Table is editable.
AFXTABLE_EDITABLE: int = hash("AFXTABLE_EDITABLE")

#: Default table options--use extended selection mode, columns are resizable, and layout fills both X and Y directions.
AFXTABLE_NORMAL: int = hash("AFXTABLE_NORMAL")

#: Do not pre-select any list item.
AFXTABLE_LIST_PRESELECT_NONE: int = hash("AFXTABLE_LIST_PRESELECT_NONE")

#: Normally list items
AFXTABLE_LIST_NORMAL: int = hash("AFXTABLE_LIST_NORMAL")


class AFXTable(AFXBaseTable, AFXDataComponent):
    """This class implements an editable table."""

    #: Popup not displayed.
    POPUP_NONE: int = hash("POPUP_NONE")

    #: Display "Cut" menu item.
    POPUP_CUT: int = hash("POPUP_CUT")

    #: Display "Copy" menu item.
    POPUP_COPY: int = hash("POPUP_COPY")

    #: Display "Paste" menu item.
    POPUP_PASTE: int = hash("POPUP_PASTE")

    #: Convenience flag for specifying multiple menu items.
    POPUP_EDIT: int = hash("POPUP_EDIT")

    #: Display "Insert Row Before/After" menu items.
    POPUP_INSERT_ROW: int = hash("POPUP_INSERT_ROW")

    #: Display "Insert Column Before/After" menu items.
    POPUP_INSERT_COLUMN: int = hash("POPUP_INSERT_COLUMN")

    #: Display "Delete Rows" menu item.
    POPUP_DELETE_ROW: int = hash("POPUP_DELETE_ROW")

    #: Display "Delete Columns" menu item.
    POPUP_DELETE_COLUMN: int = hash("POPUP_DELETE_COLUMN")

    #: Display "Clear Contents" and "Clear Table" menu items.
    POPUP_CLEAR_CONTENTS: int = hash("POPUP_CLEAR_CONTENTS")

    #: Convenience flag for specifying multiple menu items.
    POPUP_MODIFY_ROW: int = hash("POPUP_MODIFY_ROW")

    #: Convenience flag for specifying multiple menu items.
    POPUP_MODIFY_COLUMN: int = hash("POPUP_MODIFY_COLUMN")

    #: Convenience flag for specifying multiple menu items.
    POPUP_MODIFY: int = hash("POPUP_MODIFY")

    #: Display "Read from File" menu item.
    POPUP_READ_FROM_FILE: int = hash("POPUP_READ_FROM_FILE")

    #: Display "Write to File" menu item.
    POPUP_WRITE_TO_FILE: int = hash("POPUP_WRITE_TO_FILE")

    #: Display "Read from file" and "Write to file" menu items.
    POPUP_FILE: int = hash("POPUP_FILE")

    #: Display all menu items.
    POPUP_ALL: int = hash("POPUP_ALL")

    #: ID for the Cut button.
    ID_CUT_SEL: int = hash("ID_CUT_SEL")

    #: ID for the Copy button.
    ID_COPY_SEL: int = hash("ID_COPY_SEL")

    #: ID for the Paste button.
    ID_PASTE_SEL: int = hash("ID_PASTE_SEL")

    #: ID for the Insert Column buttons.
    ID_ADD_COLUMN: int = hash("ID_ADD_COLUMN")

    #: ID for the Insert Row buttons.
    ID_ADD_ROW: int = hash("ID_ADD_ROW")

    #: ID for the Delete Columns button.
    ID_DELETE_COLUMNS: int = hash("ID_DELETE_COLUMNS")

    #: ID for the Delete Rows button.
    ID_DELETE_ROWS: int = hash("ID_DELETE_ROWS")

    #: ID for the Clear Contents button.
    ID_CLEAR_SEL: int = hash("ID_CLEAR_SEL")

    #: ID for the Clear Table button.
    ID_CLEAR_TABLE: int = hash("ID_CLEAR_TABLE")

    #: ID for the Read from File button.
    ID_READ_SEL: int = hash("ID_READ_SEL")

    #: ID for the Write to File button.
    ID_WRITE: int = hash("ID_WRITE")

    #: ID for the Read Data from ASCII File dialog box used by the Read from File button.
    ID_FILE_DB: int = hash("ID_FILE_DB")

    #: ID for the "Data were truncated" warning dialog box.
    ID_READ_WARNING: int = hash("ID_READ_WARNING")

    #: ID for the file selection dialog box used by the Write to File button.
    ID_WRITE_FILE_DB: int = hash("ID_WRITE_FILE_DB")

    #: ID for the "OK to overwrite?" warning dialog box.
    ID_CONFIRM_WRITE: int = hash("ID_CONFIRM_WRITE")

    #: not exposed in python
    IGNORE_BOTTOM_EMPTY_ROWS: int = hash("IGNORE_BOTTOM_EMPTY_ROWS")

    #: Color item has no As Is and Default in its flyout.
    COLOR_INCLUDE_COLOR_ONLY: int = hash("COLOR_INCLUDE_COLOR_ONLY")

    #: Color item has As Is in its flyout.
    COLOR_INCLUDE_AS_IS: int = hash("COLOR_INCLUDE_AS_IS")

    #: Color item has Default in its flyout.
    COLOR_INCLUDE_DEFAULT: int = hash("COLOR_INCLUDE_DEFAULT")

    #: Color item has both As Is and Default in its flyout. This is the default option.
    COLOR_INCLUDE_ALL: int = hash("COLOR_INCLUDE_ALL")

    #: Disallow an item to be empty (default).
    DISALLOW_EMPTY: int = hash("DISALLOW_EMPTY")

    #: Allow an item to be empty.
    ALLOW_EMPTY: int = hash("ALLOW_EMPTY")

    #: Allow an item to be empty and use its default value for the item.
    DEFAULT_IF_EMPTY: int = hash("DEFAULT_IF_EMPTY")

    #: Include empty rows at the bottom of the table.
    KEEP_BOTTOM_EMPTY_ROWS: int = hash("KEEP_BOTTOM_EMPTY_ROWS")

    #: Left justified.
    LEFT: int = hash("LEFT")

    #: Right justified.
    RIGHT: int = hash("RIGHT")

    #: Center justified (horizontal).
    CENTER: int = hash("CENTER")

    #: Top justified.
    TOP: int = hash("TOP")

    #: Bottom justified.
    BOTTOM: int = hash("BOTTOM")

    #: Middle justified (vertical).
    MIDDLE: int = hash("MIDDLE")

    #: Item accepts a text string via a text field.
    TEXT: int = hash("TEXT")

    #: Item accepts a floating-point number via a text field.
    FLOAT: int = hash("FLOAT")

    #: Item accepts an integer via a text field.
    INT: int = hash("INT")

    #: Item accepts input from a list.
    LIST: int = hash("LIST")

    #: Item is a boolean; displayed as an icon.
    BOOL: int = hash("BOOL")

    #: Item displays an icon and does not accept input.
    ICON: int = hash("ICON")

    #: Item accepts color selection via a color flyout.
    COLOR: int = hash("COLOR")

    #: General.
    GENERAL: int = hash("GENERAL")

    #: Scientific.
    SCIENTIFIC: int = hash("SCIENTIFIC")

    #: Automatic.
    AUTOMATIC: int = hash("AUTOMATIC")

    #: Sort currently not active for the column.
    SORT_INACTIVE: int = hash("SORT_INACTIVE")

    #: Sort items of the column in the ascending order.
    SORT_ASCENDING: int = hash("SORT_ASCENDING")

    #: Sort items of the column in the descending order.
    SORT_DESCENDING: int = hash("SORT_DESCENDING")

    def __init__(
        self,
        p: FXComposite,
        numVisRows: int,
        numVisColumns: int,
        numRows: int,
        numColumns: int,
        tgt: FXObject | None = None,
        sel: int = 0,
        opts: int = AFXTABLE_NORMAL,
        x: int = 0,
        y: int = 0,
        w: int = 0,
        h: int = 0,
        pl: int = 4,
        pr: int = 4,
        pt: int = DEFAULT_MARGIN,
        pb: int = DEFAULT_MARGIN,
    ):
        """Constructor.

        Parameters
        ----------
        p : FXComposite
            Parent widget.
        numVisRows : int
            Number of rows to display.
        numVisColumns : int
            Number of columns to display.
        numRows : int
            Total number of rows including leading rows.
        numColumns : int
            Total number of columns including leading columns.
        tgt : FXObject | None
            Message target.
        sel : int
            Message ID.
        opts : int
            Options and hints.
        x : int
            X coordinate of the origin.
        y : int
            Y coordinate of the origin.
        w : int
            Width of the table widget.
        h : int
            Height of the table widget.
        pl : int
            Left padding (margin).
        pr : int
            Right padding (margin).
        pt : int
            Top padding (margin).
        pb : int
            Bottom padding (margin).
        """

    def addList(self, opts: int = AFXTABLE_LIST_NORMAL):
        """Adds a list to the table and returns the list ID. The list is used by items of type LIST.

        Parameters
        ----------
        opts : int
            List flag.
        """

    def appendListItem(self, listId: int, text: str, icon: FXIcon | None = None):
        """Appends an item to the specified table list; returns the index of the new item.

        Parameters
        ----------
        listId : int
            ID of the list to append to.
        text : str
            Item's text.
        icon : FXIcon | None
            Item's icon.
        """

    def beginEdit(self, row: int, column: int):
        """Sets the specified item in edit mode if the item is editable.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def cancelEdit(self):
        """Cancels the edit mode."""

    def clearContents(
        self, startRow: int, startColumn: int, endRow: int, endColumn: int, clearEditableOnly: bool = True
    ):
        """Clears the text in the items in the specified range.

        Parameters
        ----------
        startRow : int
            Row in which to start clearing.
        startColumn : int
            Column in which to start clearing.
        endRow : int
            Row in which to end clearing.
        endColumn : int
            Column in which to end clearing.
        clearEditableOnly : bool
            Specify True to clear the text of editable items only.
        """

    def clearListItems(self, listId: int):
        """Removes all items from the specified table list.

        Parameters
        ----------
        listId : int
            ID of the list to clear.
        """

    def create(self):
        """Creates server-side resources.

        Reimplemented from AFXBaseTable.
        """

    def deleteColumns(self, startColumn: int, numColumns: int = 1, notify: bool = False):
        """Deletes columns starting at the specified column.

        Parameters
        ----------
        startColumn : int
            Starting column.
        numColumns : int
            Number of columns to delete.
        notify : bool
            Specify True to notify target of the deletion.
        """

    def deleteRows(self, startRow: int, numRows: int = 1, notify: bool = False):
        """Deletes rows starting at the specified row.

        Parameters
        ----------
        startRow : int
            Starting row.
        numRows : int
            Number of rows to delete.
        notify : bool
            Specify True to notify target of the deletion.
        """

    def deselectItem(self, row: int, column: int):
        """Deselects the specified item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def deselectRow(self, row: int):
        """Deselects all items in the row.

        Parameters
        ----------
        row : int
            Row index.
        """

    def destroy(self):
        """Destroys server-side resources.

        Reimplemented from FXComposite.
        """

    def detach(self):
        """Detaches server-side resources.

        Reimplemented from AFXBaseTable.
        """

    def disable(self):
        """Disables the table and the table items in the table.

        Reimplemented from FXWindow.
        """

    def disableItem(self, row: int, column: int):
        """Disables the specified item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def enable(self):
        """Enables the table and the table items in the table.

        Reimplemented from FXWindow.
        """

    def enableItem(self, row: int, column: int):
        """Enables the specified item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getColumnAtX(self, x: int):
        """Returns the column at the specified x coordinate; returns -1 if x is outside of the table.

        Parameters
        ----------
        x : int
            X coordinate.
        """

    def getColumnSortOrder(self, column: int):
        """Returns the sort order of the given column.

        Parameters
        ----------
        column : int
            Column index.
        """

    def getColumnWidth(self, column: int):
        """Returns the width, in pixels, of the specified column.

        Parameters
        ----------
        column : int
            Column index.
        """

    def getCurrentColumn(self):
        """Returns the column index of the current item.

        Reimplemented from AFXBaseTable.
        """

    def getCurrentRow(self):
        """Returns the row index of the current item.

        Reimplemented from AFXBaseTable.
        """

    def getCurrentSortColumn(self):
        """Returns the current sort column or -1 if none."""

    def getDefaultColumnWidth(self):
        """Returns the default column width, in pixels, of the table."""

    def getDefaultHeight(self):
        """Returns the default height of the table.

        Reimplemented from AFXBaseTable.
        """

    def getDefaultRowHeight(self):
        """Returns the default row height, in pixels, of the table."""

    def getDefaultWidth(self):
        """Returns the default width of the table.

        Reimplemented from AFXBaseTable.
        """

    def getFont(self):
        """Gets the font for all text items in the table.

        Reimplemented from AFXBaseTable.
        """

    def getGridColor(self):
        """Gets the color of the gridlines in the table.

        Reimplemented from AFXBaseTable.
        """

    def getItemBackColor(self, row: int, column: int):
        """Gets the background color of an item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemBoolValue(self, row: int, column: int):
        """Returns the value of an item of type BOOL.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemColor(self, row: int, column: int):
        """Returns the color of an item of type COLOR. The color is "As is", "Default", or a color hex
        specification in the form of "RRGGBB" (e.g., "#0A1B2C").

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemFloatValue(self, row: int, column: int):
        """Returns the value of an item of type FLOAT.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemIcon(self, row: int, column: int):
        """Returns the icon of an item of type ICON.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemIntValue(self, row: int, column: int):
        """Returns the value of an item of type INT.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemListId(self, row: int, column: int):
        """Returns the list ID of an item of type LIST.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemListIndex(self, row: int, column: int):
        """Returns the list index (selection) of an item of type LIST.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemProvider(self):
        """Returns the item provider of this object."""

    def getItemSelector(self, row: int, column: int):
        """Returns the message ID for an item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemTarget(self, row: int, column: int):
        """Returns the target for an item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemText(self, row: int, column: int):
        """Returns the text of an item of type TEXT.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemTextColor(self, row: int, column: int):
        """Returns the text color of an item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemType(self, row: int, column: int):
        """Returns the type of an item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getItemValue(self, row: int, column: int):
        """Returns the text-form value of an item of any type.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def getLeadingColumns(self):
        """Returns the number of leading columns in the table.

        Reimplemented from AFXBaseTable.
        """

    def getLeadingFont(self):
        """Returns the font of the leading rows and columns."""

    def getLeadingRows(self):
        """Returns the number of leading rows in the table.

        Reimplemented from AFXBaseTable.
        """

    def getListItemIcon(self, listId: int, index: int):
        """Returns the icon of the item at the specified index of the specified table list.

        Parameters
        ----------
        listId : int
            ID of the list.
        index : int
            Index into the list of the item to return.
        """

    def getListItemText(self, listId: int, index: int):
        """Returns the text of the item at the specified index of the specified table list.

        Parameters
        ----------
        listId : int
            ID of the list.
        index : int
            Index into the list of the item to return.
        """

    def getNumColumns(self):
        """Returns the number of columns in the table (including leading columns).

        Reimplemented from AFXBaseTable.
        """

    def getNumEmptyRowsAtBottom(self):
        """Returns the number of empty (non-trailing) rows at the bottom of the table."""

    def getNumListItems(self, listId: int):
        """Returns the number of items in the specified table list.

        Parameters
        ----------
        listId : int
            ID of the list.
        """

    def getNumRows(self):
        """Returns the number of rows in the table (including leading rows).

        Reimplemented from AFXBaseTable.
        """

    def getPreferredColumnWidth(self, column: int, excludeTitle: bool = True):
        """Returns the width required for a column.

        Parameters
        ----------
        column : int
            Column index.
        excludeTitle : bool
            Specify True to ignore the width of leading and trailing items of the column.
        """

    def getPreferredRowHeight(self, row: int):
        """Returns the height required for a row (useful for multi-line labels).

        Parameters
        ----------
        row : int
            Row index.
        """

    def getRowAtY(self, y: int):
        """Returns the row at the specified y coordinate; returns -1 if y is outside of the table.

        Parameters
        ----------
        y : int
            Y coordinate.
        """

    def getRowHeight(self, row: int):
        """Returns the height, in pixels, of the specified row.

        Parameters
        ----------
        row : int
            Row index.
        """

    def getSelBackColor(self):
        """Gets the selection background color of the table.

        Reimplemented from AFXBaseTable.
        """

    def getSelTextColor(self):
        """Gets the selection text color of the table.

        Reimplemented from AFXBaseTable.
        """

    def getStretchableColumn(self):
        """Returns the index of the stretchable column."""

    def getTableStyle(self):
        """Returns the options related only to the table."""

    def getVisibleColumns(self):
        """Gets the number of visible columns in the table.

        Reimplemented from AFXBaseTable.
        """

    def getVisibleRows(self):
        """Gets the number of visible rows in the table.

        Reimplemented from AFXBaseTable.
        """

    def insertColumns(self, startColumn: int, numColumns: int = 1, notify: bool = False):
        """Inserts columns at the specified location.

        Parameters
        ----------
        startColumn : int
            Starting column.
        numColumns : int
            Number of columns to insert.
        notify : bool
            Specify True to notify target of the insertion.
        """

    def insertRows(self, startRow: int, numRows: int = 1, notify: bool = False):
        """Inserts rows at the specified location.

        Parameters
        ----------
        startRow : int
            Starting row.
        numRows : int
            Number of rows to insert.
        notify : bool
            Specify True to notify target of the insertion.
        """

    def isAnyItemInColumnSelected(self, column: int):
        """Returns True if any item in the column is selected.

        Parameters
        ----------
        column : int
            Column index.
        """

    def isAnyItemInRowSelected(self, row: int):
        """Returns True if any item in the row is selected.

        Parameters
        ----------
        row : int
            Row index.
        """

    def isColumnSelected(self, column: int):
        """Returns True if all items in the column are selected.

        Parameters
        ----------
        column : int
            Column index.
        """

    def isColumnSortable(self, column: int):
        """Returns True if the items of the given column can be sorted.

        Parameters
        ----------
        column : int
            Column index.
        """

    def isItemBool(self, row: int, column: int):
        """Returns True if the specified item is of type BOOL.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def isItemColor(self, row: int, column: int):
        """Returns True if the specified item is of type COLOR.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def isItemEditable(self, row: int, column: int):
        """Returns True if the item is editable.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def isItemEmpty(self, row: int, column: int):
        """Returns True if the specified item does not have a value. This method checks the actual contents of
        the specified item and does not account for the empty-item policy for the item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def isItemIcon(self, row: int, column: int):
        """Returns True if the specified item is of type ICON.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def isItemList(self, row: int, column: int):
        """Returns True if the specified item is of type LIST.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def isItemSelected(self, row: int, column: int):
        """Returns True if the specified item is selected.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def isItemText(self, row: int, column: int):
        """Returns True if the specified item is of type TEXT.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def isItemVisible(self, row: int, column: int):
        """Returns True if the specified item is visible.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def isRowSelected(self, row: int):
        """Returns True if all items in the row are selected.

        Parameters
        ----------
        row : int
            Row index.
        """

    def killFocus(self):
        """Removes the focus from this window.

        Reimplemented from AFXBaseTable.
        """

    def killSelection(self, notify: bool = False):
        """Deselects all items of the table; returns True if this method deselects any items that were selected.

        Parameters
        ----------
        notify : bool
            Specify True to notify target of the selection change (with a SEL_DESELECTED message).
        """

    def layout(self):
        """Lays out the table contents.

        Reimplemented from AFXBaseTable.
        """

    def makePositionVisible(self, row: int, column: int):
        """Scrolls to make the specified row, column fully visible.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def makeRowVisible(self, row: int):
        """Scrolls vertically (only) to make the specified row fully visible.

        Parameters
        ----------
        row : int
            Row index.
        """

    def moveContents(self, x: int, y: int):
        """Scrolls the contents.

        Parameters
        ----------
        x : int
            Distance scrolled in X direction.
        y : int
            Distance scrolled in Y direction.
        """

    def recalc(self):
        """Propagates size changes.

        Reimplemented from AFXBaseTable.
        """

    def removeListItem(self, listId: int, index: int):
        """Removes the item at the specified index from the specified table list; returns the number of items
        remaining in list.

        Parameters
        ----------
        listId : int
            ID of the list to remove from.
        index : int
            Index of the list item to remove.
        """

    def selectItem(self, row: int, column: int):
        """Selects the specified item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def selectRow(self, row: int):
        """Selects all items in the row.

        Parameters
        ----------
        row : int
            Row index.
        """

    def setColumnBoolIcons(self, column: int, trueIcon: FXIcon | None = None, falseIcon: FXIcon | None = None):
        """Sets the True and False icons of all existing and future items in a column of type BOOL. Specifying
        -1 for the column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        trueIcon : FXIcon | None
            Icon displayed when value is True; 0 = default icon.
        falseIcon : FXIcon | None
            Icon displayed when value is False; 0 = default icon.
        """

    def setColumnBoolValue(self, column: int, value: bool):
        """Sets the value of all existing and future items in a column of type BOOL. Specifying -1 for the
        column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        value : bool
            Specify True or False.
        """

    def setColumnColor(self, column: int, color: str):
        """Sets the color of all existing and future items in a column of type COLOR. The color can be "As is",
        "Default", a color hex specification in the form of "RRGGBB" (e.g., "#0A1B2C"), or a pre-defined color
        name (e.g., "Red"). Specifying -1 for the column will change all columns in the table and set the
        default for the table.

        Parameters
        ----------
        column : int
            Column index.
        color : str
            Color.
        """

    def setColumnColorItemDefault(self, column: int, color: str):
        """Sets the color of the color item in the flyout menu for all existing and future items that display
        "As is" or "Default" in a column of type COLOR. The color is either a color hex specification in the
        form of "RRGGBB" (e.g., "#0A1B2C") or a pre-defined color name (e.g., "Red"). Specifying -1 for the
        column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        color : str
            Color.
        """

    def setColumnColorOptions(self, column: int, opts: int):
        """Sets the color flyout options for all existing and future items in a column of type COLOR. Specifying
        -1 for the column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        opts : int
            Options (see ColorFlyoutOptions).
        """

    def setColumnEditable(self, column: int, editable: bool):
        """Sets the editability of all existing and future items in a column. Specifying -1 for the column will
        change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        editable : bool
            Specify True for editable, False for read-only.
        """

    def setColumnFloatValue(self, column: int, value: float):
        """Sets the value of all existing and future items in a column of type FLOAT. Specifying -1 for the
        column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        value : float
            Floating-point value.
        """

    def setColumnIcon(self, column: int, icon: FXIcon | None = None):
        """Sets the icon of all existing and future items in a column of type ICON. Specifying -1 for the column
        will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        icon : FXIcon | None
            Icon.
        """

    def setColumnIntValue(self, column: int, value: int):
        """Sets the value of all existing and future items in a column of type INT. Specifying -1 for the column
        will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        value : int
            Integer value.
        """

    def setColumnJustify(self, column: int, justify: int):
        """Sets the justification of all existing and future items in a column. Specifying -1 for the column
        will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        justify : int
            Justification (see ItemJustify).
        """

    def setColumnListId(self, column: int, listId: int):
        """Sets the list ID of all existing and future items in a column of type LIST. Specifying -1 for the
        column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        listId : int
            List ID.
        """

    def setColumnListIndex(self, column: int, index: int):
        """Sets the list index (selection) of all existing and future items in a column of type LIST. Specifying
        -1 for the column will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        index : int
            Index of item to be selected.
        """

    def setColumnSortable(self, column: int, sortable: bool):
        """Sets a column to be sortable or not. Specifying -1 for the column will change all columns in the
        table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        sortable : bool
            Specify True for sortable, False for otherwise.
        """

    def setColumnSortOrder(self, column: int, order: int):
        """Sets the sort order of the given column. Specifying -1 for the column will change all columns in the
        table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        order : int
            Sort order (see SortOrder).
        """

    def setColumnText(self, column: int, text: str):
        """Sets the text of all existing and future items in a column of type TEXT. Specifying -1 for the column
        will change all columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        text : str
            Text.
        """

    def setColumnType(self, column: int, type: int):
        """Sets the type of a column. Specifying -1 for the column will change all columns in the table and set
        the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        type : int
            Type (see Flags for item types).
        """

    def setColumnWidth(self, column: int, width: int):
        """Sets the width, in pixels, of the specified column. Specifying -1 for the column will change all non-
        leading and non-trailing columns in the table and set the default for the table. Specify -1 for the
        width will resize each specified column to best fit the width of the title(s) currently shown in its
        leading and trailing items.

        Parameters
        ----------
        column : int
            Column index.
        width : int
            Width in pixels.
        """

    def setColumnWidthInChars(self, column: int, numChars: int):
        """Sets the width, in number of characters, of the specified column. Specifying -1 for the column will
        change all non-leading and non-trailing columns in the table and set the default for the table.

        Parameters
        ----------
        column : int
            Column index.
        numChars : int
            Width in number of characters.
        """

    def setCurrentItem(self, row: int, column: int):
        """Sets the specified item to be the current item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        """

    def setCurrentSortColumn(self, column: int):
        """Sets the current sort column. The given column must be sortable; otherwise the current sort column
        will not change.

        Parameters
        ----------
        column : int
            Column index.
        """

    def setDefaultBoolIcons(self, trueIcon: FXIcon | None = None, falseIcon: FXIcon | None = None):
        """Sets the default True and False icons for the table (0 = default icon).

        Parameters
        ----------
        trueIcon : FXIcon | None
            Icon displayed when value is True; 0 = default icon.
        falseIcon : FXIcon | None
            Icon displayed when value is False; 0 = default icon.
        """

    def setDefaultBoolValue(self, value: bool):
        """Sets the default bool value for the table.

        Parameters
        ----------
        value : bool
            Specify True or False.
        """

    def setDefaultColor(self, color: str):
        """Sets the default color for all items of type COLOR in the table. The color can be "As is", "Default",
        a color hex specification in the form of "RRGGBB" (e.g., "#0A1B2C"), or a pre-defined color name (e.g.,
        "Red").

        Parameters
        ----------
        color : str
            Color.
        """

    def setDefaultColumnWidth(self, width: int):
        """Sets the default width, in pixels, for all columns.

        Parameters
        ----------
        width : int
            Width in pixels.
        """

    def setDefaultFloatValue(self, value: float):
        """Sets the default floating-point value for the table.

        Parameters
        ----------
        value : float
            Floating-point value.
        """

    def setDefaultIntValue(self, value: int):
        """Sets the default integer value for the table.

        Parameters
        ----------
        value : int
            Integer value.
        """

    def setDefaultJustify(self, justify: int):
        """Sets the default justification for the table.

        Parameters
        ----------
        justify : int
            Justification (see ItemJustify).
        """

    def setDefaultRowHeight(self, height: int):
        """Sets the default height, in pixels, for all rows.

        Parameters
        ----------
        height : int
            Height in pixels.
        """

    def setDefaultText(self, text: str):
        """Sets the default text for the table.

        Parameters
        ----------
        text : str
            Text.
        """

    def setDefaultType(self, type: int):
        """Sets the default type for the table.

        Parameters
        ----------
        type : int
            Type (see Flags for item types).
        """

    def setEmptyItemDefault(self, defaultVal: str):
        """Sets the default value (in text form) used for empty items of the table if its empty-item policy
        includes DEFAULT_IF_EMPTY.

        Parameters
        ----------
        defaultVal : str
            Default value in text form.
        """

    def setEmptyItemPolicy(self, policy: int):
        """Sets the policy for handling empty items of the table (see EmptyItemPolicy).

        Parameters
        ----------
        policy : int
            Flags for handling empty items (see EmptyItemPolicy).
        """

    def setFont(self, font: FXFont):
        """Sets the font for all text items in the table.

        Parameters
        ----------
        font : FXFont
            Font.
        """

    def setGridColor(self, color: FXColor):
        """Sets the color for the gridlines in the table.

        Parameters
        ----------
        color : FXColor
            Color.
        """

    def setItemBackColor(self, row: int, column: int, color: str):
        """Sets the background color of an item using a string.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        color : str
            Color name.
        """

    def setItemBoolIcons(self, row: int, column: int, trueIcon: FXIcon | None = None, falseIcon: FXIcon | None = None):
        """Sets the True and False icons of an item of type BOOL.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        trueIcon : FXIcon | None
            Icon displayed when value is True; 0 = default icon.
        falseIcon : FXIcon | None
            Icon displayed when value is False; 0 = default icon.
        """

    def setItemBoolValue(self, row: int, column: int, value: bool):
        """Sets the value of an item of type BOOL.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        value : bool
            Specify True or False.
        """

    def setItemColor(self, row: int, column: int, color: str):
        """Sets the color of an item of type COLOR. The color can be "As is", "Default", a color hex
        specification in the form of "RRGGBB" (e.g., "#0A1B2C"), or a pre-defined color name (e.g., "Red").

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        color : str
            Color.
        """

    def setItemEditable(self, row: int, column: int, editable: bool):
        """Sets the editability of an item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        editable : bool
            Specify True for editable, False for read-only.
        """

    def setItemFloatValue(self, row: int, column: int, value: float):
        """Sets the value of an item of type FLOAT.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        value : float
            Floating-point value.
        """

    def setItemIcon(self, row: int, column: int, icon: FXIcon | None = None):
        """Sets the icon of an item of type ICON.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        icon : FXIcon | None
            Icon.
        """

    def setItemIntValue(self, row: int, column: int, value: int):
        """Sets the value of an item of type INT.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        value : int
            Integer value.
        """

    def setItemJustify(self, row: int, column: int, justify: int):
        """Sets the justification of an item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        justify : int
            Justification (see ItemJustify).
        """

    def setItemListId(self, row: int, column: int, listId: int):
        """Sets the list ID of an item of type LIST.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        listId : int
            List ID.
        """

    def setItemListIndex(self, row: int, column: int, index: int):
        """Sets the list index (selection) of an item of type LIST.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        index : int
            Index of item to be selected.
        """

    def setItemProvider(self, ip: FXObject):
        """Sets the item provider of this object.

        Parameters
        ----------
        ip : FXObject
            Item provideer.
        """

    def setItemSpan(self, row: int, column: int, numRows: int, numColumns: int):
        """Sets a leading item to span more than one row or column.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        numRows : int
            Number of rows to span.
        numColumns : int
            Number of columns to span.
        """

    def setItemTarget(self, row: int, column: int, tgt: FXObject, msg: int = 0):
        """Sets the target and message ID for an item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        tgt : FXObject
            Target.
        msg : int
            Message ID.
        """

    def setItemText(self, row: int, column: int, text: str):
        """Sets the text of an item of type TEXT.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        text : str
            Text.
        """

    def setItemTextColor(self, row: int, column: int, color: str):
        """Sets the text color of an item using a string.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        color : str
            Color name.
        """

    def setItemType(self, row: int, column: int, type: int):
        """Sets the type of an item.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        type : int
            Type (see Flags for item types).
        """

    def setItemValue(self, row: int, column: int, valueText: str):
        """Sets the value of an item of any type that can interpret a text string for its value. Returns True if
        the value of the specified item is set successfully.

        Parameters
        ----------
        row : int
            Row index of item.
        column : int
            Column index of item.
        valueText : str
            Text-form value of item.
        """

    def setLeadingColumnLabels(self, str: str, column: int = 0):
        """Sets the labels of a leading column. Note: this API must be used to set the header column labels,
        otherwise labels will be overwritten by auto-numbering.

        Parameters
        ----------
        str : str
            Tab "t" delimited list, can also contain newline characters indicating that label contains multiple lines of text (e.g. "Young'snModulustPoisson'snRatio").
        column : int
            Column, this column must have previously been specified as a leading column (see setLeadingColumns).
        """

    def setLeadingColumns(self, numColumns: int):
        """Sets the number of leading columns.

        Parameters
        ----------
        numColumns : int
            Number of columns.
        """

    def setLeadingFont(self, font: FXFont):
        """Sets the font of the leading rows and columns.

        Parameters
        ----------
        font : FXFont
            Font.
        """

    def setLeadingRowLabels(self, str: str, row: int = 0):
        """Set the labels of a leading row. Note: this API must be used to set the header row labels, otherwise
        labels will be overwritten by auto-numbering.

        Parameters
        ----------
        str : str
            Tab "t" delimited list, can also contain newline characters indicating that label contains multiple lines of text (e.g. "Young'snModulustPoisson'snRatio").
        row : int
            Row, this row must have previously been specified as a leading row (see setLeadingRows).
        """

    def setLeadingRows(self, numRows: int):
        """Sets the number of leading rows.

        Parameters
        ----------
        numRows : int
            Number of rows.
        """

    def setListMaxVisible(self, maxVisible: int):
        """Sets the maximum number of visible items for all table lists.

        Parameters
        ----------
        maxVisible : int
            Maximum number of visible items.
        """

    def setRowHeight(self, row: int, height: int):
        """Sets the height, in pixels, of the specified row.

        Parameters
        ----------
        row : int
            Row index.
        height : int
            Height in pixels.
        """

    def setSelBackColor(self, color: FXColor):
        """Sets the selection background color of the table.

        Parameters
        ----------
        color : FXColor
            Color index.
        """

    def setSelTextColor(self, color: FXColor):
        """Sets the selection text color of the table.

        Parameters
        ----------
        color : FXColor
            Color index.
        """

    def setStretchableColumn(self, column: int):
        """Sets the stretchable column. (This method only works for the last column.)

        Parameters
        ----------
        column : int
            Column index.
        """

    def setTableSize(self, numRows: int, numColumns: int, notify: bool = False):
        """Sets the size of the table.

        Parameters
        ----------
        numRows : int
            Number of rows.
        numColumns : int
            Number of columns.
        notify : bool
            Specify True to notify target of change.
        """

    def setTableStyle(self, style: int):
        """Sets the table options.

        Parameters
        ----------
        style : int
            Style flag (see Flags for AFX table options).
        """

    def setVisibleColumns(self, visibleColumns: int):
        """Sets the number of visible columns in the table.

        Parameters
        ----------
        visibleColumns : int
            Number of visible columns.
        """

    def setVisibleRows(self, visibleRows: int):
        """Sets the number of visible rows in the table.

        Parameters
        ----------
        visibleRows : int
            Number of visible rows.
        """

    def shadeReadOnlyItems(self, shadeItems: bool):
        """Makes the table to use a different, typically shaded, background color for read-only items if True is
        passed to the method. The table would use the same regular background color for both editable and read-
        only items if False is passed to the method.

        Parameters
        ----------
        shadeItems : bool
            Specify True to use a different background color for read-only items.
        """

    def showHorizontalGrid(self, on: bool = True):
        """Controls the display of horizontal gridlines in the table.

        Parameters
        ----------
        on : bool
            True if gridlines should be displayed.
        """

    def showVerticalGrid(self, on: bool = True):
        """Controls the display of vertical gridlines in the table.

        Parameters
        ----------
        on : bool
            True if gridlines should be displayed.
        """



from __future__ import annotations

from typing import overload

from typing_extensions import Literal

from abqpy.decorators import abaqus_class_doc, abaqus_method_doc

from ..UtilityAndView.abaqusConstants import FILLED_CIRCLE, NONE, ON, SOLID, Boolean
from ..UtilityAndView.abaqusConstants import abaqusConstants as C
from .AreaStyle import AreaStyle
from .LineStyle import LineStyle
from .QuantityType import QuantityType as QuantityTypeType
from .SymbolStyle import SymbolStyle
from .TextStyle import TextStyle
from .XYData import XYData
from .XYPlotBase import XYPlotBase


@abaqus_class_doc
class XYPlot(XYPlotBase):
    @staticmethod
    @abaqus_method_doc
    def AreaStyle(color: str = "", fill: Boolean = ON, style: Literal[C.SOLID] = SOLID) -> AreaStyle:
        """This method creates an AreaStyle.

        .. note::
            This function can be accessed by::

                session.AreaStyle
                xyPlot.AreaStyle

        Parameters
        ----------
        color
            A String specifying the color to be used when filling an area with this AreaStyle
            object. The default value is "White".
        fill
            A Boolean specifying whether to fill the area when using this AreaStyle. The default
            value is ON.
        style
            A SymbolicConstant specifying the area pattern style to be used when filling an area
            using this AreaStyle. The default value is SOLID.

        Returns
        -------
        AreaStyle
            An AreaStyle object.

        Raises
        ------
        ColorError
        """
        areaStyle = AreaStyle(color, fill, style)
        return areaStyle

    @staticmethod
    @abaqus_method_doc
    def LineStyle(
        color: str = "",
        show: Boolean = ON,
        style: Literal[C.SOLID] = SOLID,
        thickness: float = 0,
    ) -> LineStyle:
        """This method creates a LineStyle.

        .. note::
            This function can be accessed by::

                session.LineStyle
                xyPlot.LineStyle

        Parameters
        ----------
        color
            A String specifying the color to be used when drawing a line with this LineStyle object.
            The default value is "White".
        show
            A Boolean specifying whether to draw the line when using this LineStyle. The default
            value is ON.
        style
            A SymbolicConstant specifying the line style to be used when drawing lines using this
            LineStyle. Possible values are SOLID, DASHED, DOTTED, and DOT_DASH. The default value is
            SOLID.
        thickness
            A Float specifying the line thickness in mm to be used when drawing lines using this
            LineStyle. The default value is 0.2.

        Returns
        -------
        LineStyle
            A LineStyle object.

        Raises
        ------
        ColorError
        """
        lineStyle = LineStyle(color, show, style, thickness)
        return lineStyle

    @staticmethod
    @abaqus_method_doc
    def QuantityType(
        type: Literal[
            C.NONE,
            C.ACCELERATION,
            C.ACOUSTIC_INTENSITY,
            C.ANGLE,
            C.ANGULAR_MOMENTUM,
            C.ARC_LENGTH,
            C.AREA,
            C.AREA_VELOCITY_SQUARED,
            C.BIMOMENT,
            C.CURVATURE,
            C.CORIOLIS_LOAD,
            C.DAMAGE,
            C.DAMAGE_CRITERION,
            C.DENSITY,
            C.DENSITY_ROTATIONAL_ACCELERATION,
            C.DISPLACEMENT,
            C.ECURRENT_AREA_TIME,
            C.ELECTRIC_CHARGE,
            C.ELECTRIC_CURRENT,
            C.ELECTRIC_CURRENT_AREA,
            C.ELECTRIC_POTENTIAL,
            C.ENERGY,
            C.ENERGY_DENSITY,
            C.ENERGY_RELEASE_RATE,
            C.EPOTENTIAL_GRADIENT,
            C.FREQUENCY,
            C.FORCE,
            C.FORCE_VOLUME,
            C.HEAT_FLUX,
            C.HEAT_FLUX_AREA,
            C.HEAT_FLUX_RATE,
            C.HEAT_FLUX_VOLUME,
            C.LENGTH,
            C.LINEAR_PRESSURE,
            C.LUMIN,
            C.MASS,
            C.MASS_FLOW_AREA,
            C.MASS_FLOW_AREA_RATE,
            C.MASS_FLOW_RATE,
            C.MODE_NUMBER,
            C.MOMENT,
            C.NUMBER,
            C.PATH,
            C.PHASE,
            C.POSITION,
            C.PRESSURE,
            C.PRESSURE_GRADIENT,
            C.RATE,
            C.ROTARY_INERTIA,
            C.ROTATIONAL_ACCELERATION,
            C.ROTATIONAL_VELOCITY,
            C.STATUS,
            C.STRAIN,
            C.STRAIN_RATE,
            C.STRESS,
            C.STRESS_INTENS_FACTOR,
            C.SUBSTANCE,
            C.TEMPERATURE,
            C.THICKNESS,
            C.TIME,
            C.TIME_INCREMENT,
            C.TIME_HEAT_FLUX,
            C.TIME_HEAT_FLUX_AREA,
            C.TIME_VOLUME,
            C.TIME_VOLUME_FLUX,
            C.TWIST,
            C.VELOCITY,
            C.VELOCITY_SQUARED,
            C.VOLUME,
            C.VOLUME_FLUX,
            C.VOLUME_FLUX_AREA,
            C.VOLUME_FRACTION,
        ] = NONE,
        label: str = "",
    ) -> QuantityTypeType:
        """This method creates a QuantityType object.

        .. note::
            This function can be accessed by::

                session.QuantityType
                xyPlot.QuantityType

        Parameters
        ----------
        type
            A SymbolicConstant specifying the physical dimension of the axis. Possible values are:

            - NONE.
            - ACCELERATION.
            - ACOUSTIC_INTENSITY.
            - ANGLE.
            - ANGULAR_MOMENTUM.
            - ARC_LENGTH.
            - AREA.
            - AREA_VELOCITY_SQUARED, specifying "Velocity squared per area".
            - BIMOMENT.
            - CURVATURE.
            - CORIOLIS_LOAD.
            - DAMAGE.
            - DAMAGE_CRITERION.
            - DENSITY.
            - DENSITY_ROTATIONAL_ACCELERATION, specifying "Density * Angular acceleration".
            - DISPLACEMENT.
            - ECURRENT_AREA_TIME, specifying "Time integrated electric current per area".
            - ELECTRIC_CHARGE.
            - ELECTRIC_CURRENT.
            - ELECTRIC_CURRENT_AREA, specifying "Electric current per unit area".
            - ELECTRIC_POTENTIAL.
            - ENERGY.
            - ENERGY_DENSITY.
            - ENERGY_RELEASE_RATE.
            - EPOTENTIAL_GRADIENT, specifying "Electric potential gradient".
            - FREQUENCY.
            - FORCE.
            - FORCE_VOLUME, specifying "Force per volume".
            - HEAT_FLUX.
            - HEAT_FLUX_AREA, specifying "Heat flux per area".
            - HEAT_FLUX_RATE.
            - HEAT_FLUX_VOLUME, specifying "Heat flux per volume".
            - LENGTH.
            - LINEAR_PRESSURE.
            - LUMIN, specifying "Luminous intensity".
            - MASS.
            - MASS_FLOW_AREA, specifying "Mass flow per area".
            - MASS_FLOW_AREA_RATE, specifying "Mass flow rate per area".
            - MASS_FLOW_RATE.
            - MODE_NUMBER.
            - MOMENT.
            - NUMBER.
            - PATH.
            - PHASE.
            - POSITION.
            - PRESSURE.
            - PRESSURE_GRADIENT.
            - RATE.
            - ROTARY_INERTIA.
            - ROTATIONAL_ACCELERATION.
            - ROTATIONAL_VELOCITY.
            - STATUS.
            - STRAIN.
            - STRAIN_RATE.
            - STRESS.
            - STRESS_INTENS_FACTOR, specifying "Stress intensity factor".
            - SUBSTANCE, specifying "Amount of substance".
            - TEMPERATURE.
            - THICKNESS.
            - TIME.
            - TIME_INCREMENT.
            - TIME_HEAT_FLUX, specifying "Time integrated heat flux".
            - TIME_HEAT_FLUX_AREA, specifying "Time integrated heat flux per area".
            - TIME_VOLUME, specifying "Time integrated volume".
            - TIME_VOLUME_FLUX, specifying "Time integrated volume flux per area".
            - TWIST.
            - VELOCITY.
            - VELOCITY_SQUARED.
            - VOLUME.
            - VOLUME_FLUX.
            - VOLUME_FLUX_AREA, specifying "Volume flux per area".
            - VOLUME_FRACTION.
        label
            A String specifying the label for this quantity type.

        Returns
        -------
        QuantityType
            A QuantityType object.
        """
        quantityType = QuantityTypeType(type, label)
        return quantityType

    @staticmethod
    @abaqus_method_doc
    def SymbolStyle(
        color: str = "",
        show: Boolean = ON,
        marker: Literal[
            C.CROSS,
            C.FILLED_DIAMOND,
            C.FILLED_SQUARE,
            C.POINT,
            C.HOLLOW_CIRCLE,
            C.HOLLOW_TRI,
            C.HOLLOW_SQUARE,
            C.FILLED_TRI,
            C.HOLLOW_DIAMOND,
            C.FILLED_CIRCLE,
            C.XMARKER,
        ] = FILLED_CIRCLE,
        size: float = 2,
    ) -> SymbolStyle:
        """This method creates a SymbolStyle object.

        .. note::
            This function can be accessed by::

                session.SymbolStyle
                xyPlot.SymbolStyle

        Parameters
        ----------
        color
            A String specifying the color to be used when drawing a marker with this SymbolStyle
            object. The default value is "White".
        show
            A Boolean specifying whether to draw the marker when using this SymbolStyle object. The
            default value is ON.
        marker
            A SymbolicConstant specifying the marker type be used when drawing symbols using this
            SymbolStyle object. Possible values are:

            - FILLED_CIRCLE
            - FILLED_SQUARE
            - FILLED_DIAMOND
            - FILLED_TRI
            - HOLLOW_CIRCLE
            - HOLLOW_SQUARE
            - HOLLOW_DIAMOND
            - HOLLOW_TRI
            - CROSS
            - XMARKER
            - POINT

            The default value is FILLED_CIRCLE.
        size
            A Float specifying the marker size to be used when drawing markers using this
            SymbolStyle object. The default value is 2.0.

        Returns
        -------
        SymbolStyle
            A SymbolStyle object.

        Raises
        ------
        ColorError
        """
        symbolStyle = SymbolStyle(color, show, marker, size)
        return symbolStyle

    @staticmethod
    @abaqus_method_doc
    def TextStyle(color: str = "", show: Boolean = ON, font: str = "", rotationAngle: float = 0) -> TextStyle:
        """This method creates a TextStyle.

        .. note::
            This function can be accessed by::

                session.TextStyle
                xyPlot.TextStyle

        Parameters
        ----------
        color
            A String specifying the color to be used when drawing text with this TextStyle object.
            The default value is "White".
        show
            A Boolean specifying whether to draw the text when using this TextStyle object. The
            default value is ON.
        font
            A String specifying the name of the font to be used when drawing text with this
            TextStyle object. The default value is "-*-verdana-medium-r-normal-*-*-120-*-*-p-*-*-*".
        rotationAngle
            A Float specifying the angle in degrees used for displaying the text. The default value
            is 0.0.

        Returns
        -------
        TextStyle
            A TextStyle object.

        Raises
        ------
        ColorError
        """
        textStyle = TextStyle(color, show, font, rotationAngle)
        return textStyle

    @staticmethod
    @overload
    @abaqus_method_doc
    def XYData(
        data: tuple,
        name: str = "",
        sourceDescription: str = "",
        contentDescription: str = "",
        positionDescription: str = "",
        legendLabel: str = "",
        xValuesLabel: str = "",
        yValuesLabel: str = "",
        axis1QuantityType: QuantityTypeType | None = None,
        axis2QuantityType: QuantityTypeType | None = None,
    ) -> XYData:
        """This method creates an XYData object from a sequence of **X - Y** data pairs.

        .. note::
            This function can be accessed by::

                session.XYData
                xyPlot.XYData

        Parameters
        ----------
        data
            A sequence of pairs of Floats specifying the **X - Y** data pairs.
        name
            The repository key. If the name is not supplied while creating the XYData object using
            xyPlot.XYData, a default name in the form _temp#_ is generated and the XYData object is
            temporary. (This argument is required if the method is accessed from the session
            object.)
        sourceDescription
            A String specifying the source of the **X - Y** data (e.g., “Entered from keyboard”, “Taken
            from ASCII file”, “Read from an ODB”, etc.). The default value is an empty string.
        contentDescription
            A String specifying the content of the **X - Y** data (e.g., “field 1 vs. field 2”). The
            default value is an empty string.
        positionDescription
            A String specifying additional information about the **X - Y** data (e.g., “for whole
            model”). The default value is an empty string.
        legendLabel
            A String specifying the label to be used in the legend. The default value is the name of
            the XYData object.
        xValuesLabel
            A String specifying the label for the X-values. This value may be overridden if the
            **X - Y** data are combined with other **X - Y** data. The default value is an empty string.
        yValuesLabel
            A String specifying the label for the Y-values. This value may be overridden if the
            **X - Y** data are combined with other **X - Y** data. The default value is an empty string.
        axis1QuantityType
            A QuantityType object specifying the QuantityType object associated to the X -axis1-
            values.
        axis2QuantityType
            A QuantityType object specifying the QuantityType object associated to the Y -axis2-
            values.

        Returns
        -------
        XYData
            An XYData object.
        """
        ...

    @staticmethod
    @overload
    @abaqus_method_doc
    def XYData(objectToCopy: XYData) -> XYData:
        """This method creates an XYData object by copying an existing XYData object.

        .. note::
            This function can be accessed by::

                session.odbs[name].userData.XYData
                session.XYData
                xyPlot.XYData

        Parameters
        ----------
        objectToCopy
            An XYData object to be copied.

        Returns
        -------
        XYData
            An XYData object.
        """
        ...

    @staticmethod
    def XYData(*args, **kwargs) -> XYData:
        return XYData(())



from abqpy.decorators import abaqus_class_doc

from .PlyStackPlotOptions import PlyStackPlotOptions


@abaqus_class_doc
class DetailPlotOptions:
    """The DetailPlotOptions object stores values and attributes associated with a Viewport
    object. The DetailPlotOptions object has no constructor command. Abaqus creates the
    **detailPlotOptions** member whenever a Viewport is created.

    .. note::
        This object can be accessed by::

            session.viewports[name].detailPlotOptions
    """

    #: A PlyStackPlotOptions object.
    plyStackPlotOptions: PlyStackPlotOptions = PlyStackPlotOptions()
